@@ -55,8 +55,22 @@
  */
 
 /*
- * $Id: SEnumVal.cpp,v 1.8 2002/02/20 20:30:11 peiyongz Exp $
+ * $Id: SEnumVal.cpp,v 1.12 2002/07/12 15:51:52 knoaman Exp $
  * $Log: SEnumVal.cpp,v $
+ * Revision 1.12  2002/07/12 15:51:52  knoaman
+ * Retrieve the root grammar instead of using the validator.
+ * Modify the way we print substitution group info.
+ *
+ * Revision 1.11  2002/06/25 15:30:46  peiyongz
+ * Bug#10067: SEnumVal bugs found when porting to Visual Studio .NET
+ *                     projects, patch from Robert Buck (rbuck@mathworks.com )
+ *
+ * Revision 1.10  2002/05/08 18:18:46  knoaman
+ * Fix bor bug 8301: INFINITY used as enum member.
+ *
+ * Revision 1.9  2002/04/17 20:18:08  tng
+ * [Bug 7493] The word "occured" is misspelled and it is a global error.
+ *
  * Revision 1.8  2002/02/20 20:30:11  peiyongz
  * Make the code compilable on Solaris 2.6's CC
  *
@@ -227,7 +241,7 @@ void process(char* const xmlFile)
 
     if (parser.getErrorCount())
 	{
-        cout << "\nErrors occured, no output available\n" << endl;
+        cout << "\nErrors occurred, no output available\n" << endl;
 		return;
 	}
 
@@ -237,11 +251,12 @@ void process(char* const xmlFile)
 		return;
 	}
 
-    if ( parser.getValidator().getGrammar()->getGrammarType() != Grammar::SchemaGrammarType)
-    {
+	Grammar* rootGrammar = parser.getRootGrammar();
+	if (!rootGrammar || rootGrammar->getGrammarType() != Grammar::SchemaGrammarType)
+	{
 		cout << "\n Non schema grammar, no output available\n" << endl;
 		return;
-    }
+	}
 
 	//
 	//  Now we will get an enumerator for the element pool from the validator
@@ -249,7 +264,7 @@ void process(char* const xmlFile)
 	//  we get an enumerator for its attributes and print them also.
 	//
 
-    SchemaGrammar* grammar = (SchemaGrammar*) parser.getValidator().getGrammar();
+	SchemaGrammar* grammar = (SchemaGrammar*) rootGrammar;
 	RefHash3KeysIdPoolEnumerator<SchemaElementDecl> elemEnum = grammar->getElemEnumerator();
 
 	if (!elemEnum.hasMoreElements())
@@ -307,21 +322,14 @@ void process(char* const xmlFile)
 			cout << "Misc. Flags:\t";
 		}
 
-		if( mflags == SchemaSymbols::INFINITY  )
-		{
-			cout << "Infinity ";
-		}
-		else
-		{
-			if ( mflags & SchemaSymbols::NILLABLE  != 0 )
-				cout << "Nillable ";
+        if ( mflags & SchemaSymbols::NILLABLE )
+			cout << "Nillable ";
 
-			if ( mflags & SchemaSymbols::ABSTRACT  != 0 )
-				cout << "Abstract ";
+		if ( mflags & SchemaSymbols::ABSTRACT )
+			cout << "Abstract ";
 
-			if ( mflags & SchemaSymbols::FIXED     != 0 )
-				cout << "Fixed ";
-		}
+		if ( mflags & SchemaSymbols::FIXED )
+			cout << "Fixed ";
 
 		if( mflags !=0 )
 		{
@@ -329,10 +337,12 @@ void process(char* const xmlFile)
 		}
 
 		// Substitution Name
-		XMLCh* subsGroup = curElem.getSubstitutionGroupName();
+		SchemaElementDecl* subsGroup = curElem.getSubstitutionGroupElem();
 		if( subsGroup )
 		{
-			cout << "Substitution Name:\t" << StrX(subsGroup) << "\n";
+			const XMLCh* uriText = parser.getURIText(subsGroup->getURI());
+			cout << "Substitution Name:\t" << StrX(uriText)
+			     << "," << StrX(subsGroup->getBaseName()) << "\n";
 		}
 
 		// Content Model
