@@ -56,6 +56,37 @@
 
 /*
  * $Log: SAX2XMLReaderImpl.cpp,v $
+ * Revision 1.10  2002/08/14 15:20:38  knoaman
+ * [Bug 3111] Problem with LexicalHandler::startDTD() and LexicalHandler::endDTD().
+ *
+ * Revision 1.9  2002/07/11 18:27:20  knoaman
+ * Grammar caching/preparsing - initial implementation.
+ *
+ * Revision 1.8  2002/06/17 15:41:15  tng
+ * To be consistent, SAX2 is updated with:
+ * 1. the progressive parse methods should use the fReuseGrammar flag set from setFeature instead of using parameter
+ * 2. add feature "http://apache.org/xml/features/continue-after-fatal-error", and users should use setFeature instead of setExitOnFirstFatalError
+ * 3. add feature "http://apache.org/xml/features/validation-error-as-fatal", and users should use setFeature instead of setValidationConstraintFatal
+ *
+ * Revision 1.7  2002/05/30 16:20:09  tng
+ * Add feature to optionally ignore external DTD.
+ *
+ * Revision 1.6  2002/05/29 21:37:47  knoaman
+ * Add baseURI to resolveEntity to support DOMInputSource.
+ *
+ * Revision 1.5  2002/05/28 20:44:14  tng
+ * [Bug 9104] prefixes dissapearing when schema validation turned on.
+ *
+ * Revision 1.4  2002/05/27 18:39:21  tng
+ * To get ready for 64 bit large file, use XMLSSize_t to represent line and column number.
+ *
+ * Revision 1.3  2002/05/22 20:53:41  knoaman
+ * Prepare for DOM L3 :
+ * - Make use of the XMLEntityHandler/XMLErrorReporter interfaces, instead of using
+ * EntityHandler/ErrorHandler directly.
+ * - Add 'AbstractDOMParser' class to provide common functionality for XercesDOMParser
+ * and DOMBuilder.
+ *
  * Revision 1.2  2002/02/13 16:09:24  knoaman
  * Move SAX2 features/properties names constants to XMLUni.
  *
@@ -239,9 +270,6 @@ SAX2XMLReaderImpl::SAX2XMLReaderImpl() :
 	fValidation = true;
 	fautoValidation = false;
 
-	// default: reuseValidator is off
-	fReuseGrammar = false;
-
 	// default: schema is on
 	setDoSchema(true);
 	
@@ -444,11 +472,9 @@ void SAX2XMLReaderImpl::setEntityResolver(EntityResolver* const resolver)
     fEntityResolver = resolver;
     if (fEntityResolver) {
         fScanner->setEntityHandler(this);
-        fScanner->setEntityResolver(fEntityResolver);
     }
     else {
         fScanner->setEntityHandler(0);
-        fScanner->setEntityResolver(0);
     }
 }
 
@@ -471,7 +497,7 @@ void SAX2XMLReaderImpl::parse (const   InputSource&    source)
     try
     {
         fParseInProgress = true;
-        fScanner->scanDocument(source, fReuseGrammar);
+        fScanner->scanDocument(source);
         fParseInProgress = false;
     }
 
@@ -491,7 +517,7 @@ void SAX2XMLReaderImpl::parse (const   XMLCh* const    systemId)
     try
     {
         fParseInProgress = true;
-        fScanner->scanDocument(systemId, fReuseGrammar);
+        fScanner->scanDocument(systemId);
         fParseInProgress = false;
     }
 
@@ -511,7 +537,7 @@ void SAX2XMLReaderImpl::parse (const   char* const     systemId)
     try
     {
         fParseInProgress = true;
-        fScanner->scanDocument(systemId, fReuseGrammar);
+        fScanner->scanDocument(systemId);
         fParseInProgress = false;
     }
 
@@ -526,8 +552,7 @@ void SAX2XMLReaderImpl::parse (const   char* const     systemId)
 //  SAX2XMLReaderImpl: Progressive parse methods
 // ---------------------------------------------------------------------------
 bool SAX2XMLReaderImpl::parseFirst( const   XMLCh* const    systemId
-                            ,       XMLPScanToken&  toFill
-                            , const bool            reuseGrammar)
+                            ,       XMLPScanToken&  toFill)
 {
     //
     //  Avoid multiple entrance. We cannot enter here while a regular parse
@@ -536,12 +561,11 @@ bool SAX2XMLReaderImpl::parseFirst( const   XMLCh* const    systemId
     if (fParseInProgress)
         ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
 
-    return fScanner->scanFirst(systemId, toFill, reuseGrammar);
+    return fScanner->scanFirst(systemId, toFill);
 }
 
 bool SAX2XMLReaderImpl::parseFirst( const   char* const     systemId
-                            ,       XMLPScanToken&  toFill
-                            , const bool            reuseGrammar)
+                            ,       XMLPScanToken&  toFill)
 {
     //
     //  Avoid multiple entrance. We cannot enter here while a regular parse
@@ -550,12 +574,11 @@ bool SAX2XMLReaderImpl::parseFirst( const   char* const     systemId
     if (fParseInProgress)
         ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
 
-    return fScanner->scanFirst(systemId, toFill, reuseGrammar);
+    return fScanner->scanFirst(systemId, toFill);
 }
 
 bool SAX2XMLReaderImpl::parseFirst( const   InputSource&    source
-                            ,       XMLPScanToken&  toFill
-                            , const bool            reuseGrammar)
+                            ,       XMLPScanToken&  toFill)
 {
     //
     //  Avoid multiple entrance. We cannot enter here while a regular parse
@@ -564,7 +587,7 @@ bool SAX2XMLReaderImpl::parseFirst( const   InputSource&    source
     if (fParseInProgress)
         ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
 
-    return fScanner->scanFirst(source, toFill, reuseGrammar);
+    return fScanner->scanFirst(source, toFill);
 }
 
 bool SAX2XMLReaderImpl::parseNext(XMLPScanToken& token)
@@ -764,116 +787,116 @@ startElement(   const   XMLElementDecl&         elemDecl
                 , const bool                    isEmpty
                 , const bool                    isRoot)
 {
-
-	// Bump the element depth counter if not empty
+    // Bump the element depth counter if not empty
     if (!isEmpty)
         fElemDepth++;
 
     if (fDocHandler)
-	{
-	
-		if (getDoNamespaces())
-		{
+    {
+        XMLBufBid elemQName( &fStringBuffers ) ;
+        if (elemPrefix && *elemPrefix) {
+            elemQName.set(elemPrefix);
+            elemQName.append(chColon);
+        }
+        elemQName.append(elemDecl.getBaseName());
+
+        if (getDoNamespaces())
+        {
             unsigned int numPrefix = 0;
-			const XMLCh*   nsString = XMLUni::fgXMLNSString;
-			const XMLCh*   nsPrefix = 0;
-			const XMLCh*   nsURI    = 0;
-			const XMLAttr* tempAttr = 0;
+            const XMLCh*   nsString = XMLUni::fgXMLNSString;
+            const XMLCh*   nsPrefix = 0;
+            const XMLCh*   nsURI    = 0;
+            const XMLAttr* tempAttr = 0;
 
             if (!fnamespacePrefix)
-			{
+            {
                 tempAttrVec->removeAllElements();
-			}
+            }
 
-			for (unsigned int i = 0; i < attrCount; i++)
-			{
-				tempAttr = attrList.elementAt(i);
-                  if (XMLString::compareString(tempAttr->getQName(), nsString) == 0)
-					nsURI = tempAttr->getValue();
-                  if (XMLString::compareString(tempAttr->getPrefix(), nsString) == 0)
-				{
-					nsPrefix = tempAttr->getName();
-					nsURI = tempAttr->getValue();
-				}
-				if (!fnamespacePrefix)
-				{
-					if (nsURI == 0)
-						tempAttrVec->addElement((XMLAttr* const)tempAttr);
-				}
-				if (nsURI != 0)
-				{
-					if (nsPrefix == 0)
-						nsPrefix = XMLUni::fgZeroLenString;
-					fDocHandler->startPrefixMapping(nsPrefix, nsURI);
-					
-					XMLBuffer &buf = fStringBuffers.bidOnBuffer()  ;
-					buf.set ( nsPrefix ) ;
-					fPrefixes->push(&buf) ;
-
-					numPrefix++;
-				}
-				nsURI = 0;
-				nsPrefix = 0;	
-			}
-			prefixCounts->push(numPrefix) ;
-			if (!fnamespacePrefix)
-				fAttrList.setVector(tempAttrVec, tempAttrVec->size(), fScanner);
-			else
+            for (unsigned int i = 0; i < attrCount; i++)
+            {
+                tempAttr = attrList.elementAt(i);
+                if (XMLString::compareString(tempAttr->getQName(), nsString) == 0)
+                    nsURI = tempAttr->getValue();
+                if (XMLString::compareString(tempAttr->getPrefix(), nsString) == 0)
+                {
+                    nsPrefix = tempAttr->getName();
+                    nsURI = tempAttr->getValue();
+                }
+                if (!fnamespacePrefix)
+                {
+                    if (nsURI == 0)
+                        tempAttrVec->addElement((XMLAttr* const)tempAttr);
+                }
+                if (nsURI != 0)
+                {
+                    if (nsPrefix == 0)
+                        nsPrefix = XMLUni::fgZeroLenString;
+                    fDocHandler->startPrefixMapping(nsPrefix, nsURI);
+                    XMLBuffer &buf = fStringBuffers.bidOnBuffer();
+                    buf.set ( nsPrefix ) ;
+                    fPrefixes->push(&buf) ;
+                    numPrefix++;
+                }
+                nsURI = 0;
+                nsPrefix = 0;
+            }
+            prefixCounts->push(numPrefix) ;
+            if (!fnamespacePrefix)
+                fAttrList.setVector(tempAttrVec, tempAttrVec->size(), fScanner);
+            else
                 fAttrList.setVector(&attrList, attrCount, fScanner);
 
-			// call startElement() with namespace declarations
-			XMLBufBid URIBufferBid ( &fStringBuffers ) ;
-			XMLBuffer &URIBuffer = URIBufferBid.getBuffer() ;
+            // call startElement() with namespace declarations
+            XMLBufBid URIBufferBid ( &fStringBuffers ) ;
+            XMLBuffer &URIBuffer = URIBufferBid.getBuffer() ;
 
-			fScanner->getURIText(elemURLId, (XMLBuffer &)URIBuffer);
-			
-			fDocHandler->startElement(URIBuffer.getRawBuffer(),
+            fScanner->getURIText(elemURLId, (XMLBuffer &)URIBuffer);
+
+            fDocHandler->startElement(URIBuffer.getRawBuffer(),
+										elemDecl.getBaseName(),
+										elemQName.getRawBuffer(),
+										fAttrList);
+        }
+        else // no namespace
+        {
+            fAttrList.setVector(&attrList, attrCount, fScanner);
+            fDocHandler->startElement(XMLUni::fgZeroLenString,
 										elemDecl.getBaseName(),
 										elemDecl.getFullName(),
 										fAttrList);
-		}
-		else
-		{
-			fAttrList.setVector(&attrList, attrCount, fScanner);
-			fDocHandler->startElement(XMLUni::fgZeroLenString,
-										elemDecl.getBaseName(),
-										elemDecl.getFullName(),
-										fAttrList);
-		}
+        }
 
 
         // If its empty, send the end tag event now
         if (isEmpty)
-		{
+        {
+            // call endPrefixMapping appropriately.
+            if (getDoNamespaces())
+            {
+                XMLBufBid URIBufferBid ( &fStringBuffers ) ;
+                XMLBuffer &URIBuffer = URIBufferBid.getBuffer() ;
+                fScanner->getURIText(elemURLId, (XMLBuffer &)URIBuffer);
 
-			// call endPrefixMapping appropriately.
-			if (getDoNamespaces())
-			{
-				XMLBufBid URIBufferBid ( &fStringBuffers ) ;
-				XMLBuffer &URIBuffer = URIBufferBid.getBuffer() ;
-
-				fScanner->getURIText(elemURLId, (XMLBuffer &)URIBuffer);
-			
-				fDocHandler->endElement(	URIBuffer.getRawBuffer(),
+                fDocHandler->endElement(	URIBuffer.getRawBuffer(),
 											elemDecl.getBaseName(),
-											elemDecl.getFullName() );
-				unsigned int numPrefix = prefixCounts->pop();
-				for (unsigned int i = 0; i < numPrefix; ++i)
-				{
-					XMLBuffer * buf = fPrefixes->pop() ;
-					fDocHandler->endPrefixMapping( buf->getRawBuffer() );
-					fStringBuffers.releaseBuffer(*buf) ;
-				}
-			}
-			else
-			{
-				fDocHandler->endElement(XMLUni::fgZeroLenString,
-							elemDecl.getBaseName(),
-							elemDecl.getFullName() );
+											elemQName.getRawBuffer());
 
-			}
-
-		}
+                unsigned int numPrefix = prefixCounts->pop();
+                for (unsigned int i = 0; i < numPrefix; ++i)
+                {
+                    XMLBuffer * buf = fPrefixes->pop() ;
+                    fDocHandler->endPrefixMapping( buf->getRawBuffer() );
+                    fStringBuffers.releaseBuffer(*buf) ;
+                }
+            }
+            else
+            {
+                fDocHandler->endElement(XMLUni::fgZeroLenString,
+                                elemDecl.getBaseName(),
+                                elemDecl.getFullName());
+            }
+        }
     }
 
     //
@@ -897,46 +920,54 @@ startElement(   const   XMLElementDecl&         elemDecl
 
 void SAX2XMLReaderImpl::endElement( const   XMLElementDecl& elemDecl
                             , const unsigned int    uriId
-                            , const bool            isRoot)
+                            , const bool            isRoot
+                            , const XMLCh* const    elemPrefix)
 {
     // Just map to the SAX document handler
     if (fDocHandler)
-	{
+    {
 
-		// get the prefixes back so that we can call endPrefixMapping()
+        // get the prefixes back so that we can call endPrefixMapping()
         if (getDoNamespaces())
-		{
-			XMLBufBid URIBufferBid ( &fStringBuffers ) ;
-			XMLBuffer &URIBuffer = URIBufferBid.getBuffer() ;
+        {
+            XMLBufBid URIBufferBid ( &fStringBuffers ) ;
+            XMLBuffer &URIBuffer = URIBufferBid.getBuffer() ;
 
-			fScanner->getURIText(uriId, URIBuffer ) ;
-			fDocHandler->endElement(	URIBuffer.getRawBuffer(),
-										elemDecl.getBaseName(),
-										elemDecl.getFullName() );
+            fScanner->getURIText(uriId, URIBuffer ) ;
 
-			unsigned int numPrefix = prefixCounts->pop();
-			for (unsigned int i = 0; i < numPrefix; i++)
-			{
-				XMLBuffer * buf = fPrefixes->pop() ;
-				fDocHandler->endPrefixMapping( buf->getRawBuffer() );
-				fStringBuffers.releaseBuffer(*buf) ;
-			}
-		}
-		else
-		{
-			fDocHandler->endElement(XMLUni::fgZeroLenString,
+            XMLBufBid elemQName( &fStringBuffers ) ;
+            if (elemPrefix && *elemPrefix) {
+                elemQName.set(elemPrefix);
+                elemQName.append(chColon);
+            }
+            elemQName.append(elemDecl.getBaseName());
+
+            fDocHandler->endElement(	URIBuffer.getRawBuffer(),
 										elemDecl.getBaseName(),
-										elemDecl.getFullName() );
-		}
-	
-	}
+										elemQName.getRawBuffer());
+
+            unsigned int numPrefix = prefixCounts->pop();
+            for (unsigned int i = 0; i < numPrefix; i++)
+            {
+                XMLBuffer * buf = fPrefixes->pop() ;
+                fDocHandler->endPrefixMapping( buf->getRawBuffer() );
+                fStringBuffers.releaseBuffer(*buf) ;
+            }
+        }
+        else
+        {
+            fDocHandler->endElement(XMLUni::fgZeroLenString,
+            elemDecl.getBaseName(),
+            elemDecl.getFullName() );
+        }
+    }
 
     //
     //  If there are any installed advanced handlers, then lets call them
     //  with this info.
     //
     for (unsigned int index = 0; index < fAdvDHCount; index++)
-        fAdvDHList[index]->endElement(elemDecl, uriId, isRoot);
+        fAdvDHList[index]->endElement(elemDecl, uriId, isRoot, elemPrefix);
 
     //
     //  Dump the element depth down again. Don't let it underflow in case
@@ -1005,7 +1036,7 @@ void SAX2XMLReaderImpl::attDef( const   DTDElementDecl& elemDecl
 
         fDeclHandler->attributeDecl(elemDecl.getFullName(),
                                     attDef.getFullName(),
-                                    (isEnumeration) ? enumBuf.getRawBuffer() 
+                                    (isEnumeration) ? enumBuf.getRawBuffer()
                                                     : attDef.getAttTypeString(attDef.getType()),
                                     defAttTypeStr,
                                     attDef.getValue());
@@ -1027,12 +1058,15 @@ void SAX2XMLReaderImpl::doctypeComment(const XMLCh* const commentText)
 void SAX2XMLReaderImpl::doctypeDecl(const   DTDElementDecl& elemDecl
                             , const XMLCh* const    publicId
                             , const XMLCh* const    systemId
-                            , const bool            hasIntSubset)
+                            , const bool            hasIntSubset
+                            , const bool            hasExtSubset)
 {
-   // Call the installed LexicalHandler.
-   if (fLexicalHandler)
+    // Call the installed LexicalHandler.
+    if (fLexicalHandler && (hasIntSubset || hasExtSubset))
         fLexicalHandler->startDTD(elemDecl.getFullName(), publicId, systemId);
 
+    fHasExternalSubset = hasExtSubset;
+
     // Unused by SAX DTDHandler interface at this time
 }
 
@@ -1072,7 +1106,7 @@ void SAX2XMLReaderImpl::endIntSubset()
    if (!fHasExternalSubset && fLexicalHandler)
         fLexicalHandler->endDTD();
 
-    // Unused by SAX DTDHandler interface at this time    
+    // Unused by SAX DTDHandler interface at this time
 }
 
 
@@ -1186,8 +1220,6 @@ void SAX2XMLReaderImpl::startIntSubset()
 
 void SAX2XMLReaderImpl::startExtSubset()
 {
-    fHasExternalSubset = true;
-
     if (fLexicalHandler)
         fLexicalHandler->startEntity(gDTDEntityStr);
 }
@@ -1220,7 +1252,8 @@ void SAX2XMLReaderImpl::resetEntities()
 
 
 InputSource* SAX2XMLReaderImpl::resolveEntity(   const   XMLCh* const    publicId
-                                               , const   XMLCh* const    systemId)
+                                               , const   XMLCh* const    systemId
+                                               , const   XMLCh* const    baseURI)
 {
     // Just map to the SAX entity resolver handler
     if (fEntityResolver)
@@ -1250,8 +1283,8 @@ void SAX2XMLReaderImpl::error(  const   unsigned int                code
                         , const XMLCh* const                errorText
                         , const XMLCh* const                systemId
                         , const XMLCh* const                publicId
-                        , const unsigned int                lineNum
-                        , const unsigned int                colNum)
+                        , const XMLSSize_t                  lineNum
+                        , const XMLSSize_t                  colNum)
 {
     SAXParseException toThrow = SAXParseException
     (
@@ -1286,90 +1319,104 @@ void SAX2XMLReaderImpl::error(  const   unsigned int                code
 void SAX2XMLReaderImpl::setFeature(const XMLCh* const name, const bool value)
 {
 
-	if (fParseInProgress)
-		throw SAXNotSupportedException("Feature modification is not supported during parse.");
+    if (fParseInProgress)
+        throw SAXNotSupportedException("Feature modification is not supported during parse.");
 	
-	if (XMLString::compareIString(name, XMLUni::fgSAX2CoreNameSpaces) == 0)
-	{
-		setDoNamespaces(value);
-	}
+    if (XMLString::compareIString(name, XMLUni::fgSAX2CoreNameSpaces) == 0)
+    {
+        setDoNamespaces(value);
+    }
+    else if (XMLString::compareIString(name, XMLUni::fgSAX2CoreValidation) == 0)
+    {
+        fValidation = value;
+        if (fValidation)
+            if (fautoValidation)
+                setValidationScheme(Val_Auto);
+            else
+                setValidationScheme(Val_Always);
+        else
+            setValidationScheme(Val_Never);
+    }
+    else if (XMLString::compareIString(name, XMLUni::fgSAX2CoreNameSpacePrefixes) == 0)
+    {
+        fnamespacePrefix = value;
+    }
+    else if (XMLString::compareIString(name, XMLUni::fgXercesDynamic) == 0)
+    {
+        fautoValidation = value;
+        // for auto validation, the sax2 core validation feature must also be enabled.
+        if (fValidation)
+            if (fautoValidation)
+                setValidationScheme(Val_Auto);
+            else
+                setValidationScheme(Val_Always);
+        else
+            setValidationScheme(Val_Never);
+    }
+    else if (XMLString::compareIString(name, XMLUni::fgXercesSchema) == 0)
+    {
+        setDoSchema(value);
+    }
+    else if (XMLString::compareIString(name, XMLUni::fgXercesSchemaFullChecking) == 0)
+    {
+        fScanner->setValidationSchemaFullChecking(value);
+    }
+    else if (XMLString::compareIString(name, XMLUni::fgXercesLoadExternalDTD) == 0)
+    {
+        fScanner->setLoadExternalDTD(value);
+    }
+    else if (XMLString::compareIString(name, XMLUni::fgXercesContinueAfterFatalError) == 0)
+    {
+        fScanner->setExitOnFirstFatal(!value);
+    }
+    else if (XMLString::compareIString(name, XMLUni::fgXercesValidationErrorAsFatal) == 0)
+    {
+        fScanner->setValidationConstraintFatal(value);
+    }
+    else if (XMLString::compareIString(name, XMLUni::fgXercesCacheGrammarFromParse) == 0)
+    {
+        fScanner->cacheGrammarFromParse(value);
 
-	else if (XMLString::compareIString(name, XMLUni::fgSAX2CoreValidation) == 0)
-	{
-		fValidation = value;
-		if (fValidation)
-			if (fautoValidation)
-				setValidationScheme(Val_Auto);
-			else
-				setValidationScheme(Val_Always);
-		else
-			setValidationScheme(Val_Never);
-	}
-
-
-	else if (XMLString::compareIString(name, XMLUni::fgSAX2CoreNameSpacePrefixes) == 0)
-	{
-		fnamespacePrefix = value;
-	}
-
-	else if (XMLString::compareIString(name, XMLUni::fgSAX2XercesDynamic) == 0)
-	{
-		fautoValidation = value;
-		// for auto validation, the sax2 core validation feature must also be enabled.
-		if (fValidation)
-			if (fautoValidation)
-				setValidationScheme(Val_Auto);
-			else
-				setValidationScheme(Val_Always);
-		else
-			setValidationScheme(Val_Never);
-	}
-
-	else if (XMLString::compareIString(name, XMLUni::fgSAX2XercesReuseValidator) == 0)
-	{
-		fReuseGrammar = value;
-	}
-
-	else if (XMLString::compareIString(name, XMLUni::fgSAX2XercesReuseGrammar) == 0)
-	{
-		fReuseGrammar = value;
-	}
-
-	else if (XMLString::compareIString(name, XMLUni::fgSAX2XercesSchema) == 0)
-	{
-		setDoSchema(value);
-	}
-
-	else if (XMLString::compareIString(name, XMLUni::fgSAX2XercesSchemaFullChecking) == 0)
-	{
-		fScanner->setValidationSchemaFullChecking(value);
-	}
-
-   else
+        if (value)
+            fScanner->useCachedGrammarInParse(value);
+    }
+    else if (XMLString::compareIString(name, XMLUni::fgXercesUseCachedGrammarInParse) == 0)
+    {
+        if (value || !fScanner->isCachingGrammarFromParse())
+            fScanner->useCachedGrammarInParse(value);
+    }
+    else
        throw SAXNotRecognizedException("Unknown Feature");
 }
 
 bool SAX2XMLReaderImpl::getFeature(const XMLCh* const name) const
 {
-	if (XMLString::compareIString(name, XMLUni::fgSAX2CoreNameSpaces) == 0)
-		return getDoNamespaces();
-	else if (XMLString::compareIString(name, XMLUni::fgSAX2CoreValidation) == 0)
-		return fValidation;
-	else if (XMLString::compareIString(name, XMLUni::fgSAX2CoreNameSpacePrefixes) == 0)
-		return fnamespacePrefix;
-	else if (XMLString::compareIString(name, XMLUni::fgSAX2XercesDynamic) == 0)
-		return fautoValidation;
-	else if (XMLString::compareIString(name, XMLUni::fgSAX2XercesReuseValidator) == 0)
-        return fReuseGrammar;
-	else if (XMLString::compareIString(name, XMLUni::fgSAX2XercesReuseGrammar) == 0)
-        return fReuseGrammar;
-	else if (XMLString::compareIString(name, XMLUni::fgSAX2XercesSchema) == 0)
+    if (XMLString::compareIString(name, XMLUni::fgSAX2CoreNameSpaces) == 0)
+        return getDoNamespaces();
+    else if (XMLString::compareIString(name, XMLUni::fgSAX2CoreValidation) == 0)
+        return fValidation;
+    else if (XMLString::compareIString(name, XMLUni::fgSAX2CoreNameSpacePrefixes) == 0)
+        return fnamespacePrefix;
+    else if (XMLString::compareIString(name, XMLUni::fgXercesDynamic) == 0)
+        return fautoValidation;
+    else if (XMLString::compareIString(name, XMLUni::fgXercesSchema) == 0)
         return getDoSchema();
-	else if (XMLString::compareIString(name, XMLUni::fgSAX2XercesSchemaFullChecking) == 0)
+    else if (XMLString::compareIString(name, XMLUni::fgXercesSchemaFullChecking) == 0)
         return fScanner->getValidationSchemaFullChecking();
-   else
+    else if (XMLString::compareIString(name, XMLUni::fgXercesLoadExternalDTD) == 0)
+        return fScanner->getLoadExternalDTD();
+    else if (XMLString::compareIString(name, XMLUni::fgXercesContinueAfterFatalError) == 0)
+        return !fScanner->getExitOnFirstFatal();
+    else if (XMLString::compareIString(name, XMLUni::fgXercesValidationErrorAsFatal) == 0)
+        return fScanner->getValidationConstraintFatal();
+    else if (XMLString::compareIString(name, XMLUni::fgXercesCacheGrammarFromParse) == 0)
+        return fScanner->isCachingGrammarFromParse();
+    else if (XMLString::compareIString(name, XMLUni::fgXercesUseCachedGrammarInParse) == 0)
+        return fScanner->isUsingCachedGrammarInParse();
+    else
        throw SAXNotRecognizedException("Unknown Feature");
-	return false;
+
+    return false;
 }
 
 void SAX2XMLReaderImpl::setProperty(const XMLCh* const name, void* value)
@@ -1377,12 +1424,12 @@ void SAX2XMLReaderImpl::setProperty(const XMLCh* const name, void* value)
 	if (fParseInProgress)
 		throw SAXNotSupportedException("Property modification is not supported during parse.");
 
-	if (XMLString::compareIString(name, XMLUni::fgSAX2XercesSchemaExternalSchemaLocation) == 0)
+	if (XMLString::compareIString(name, XMLUni::fgXercesSchemaExternalSchemaLocation) == 0)
 	{
 		fScanner->setExternalSchemaLocation((XMLCh*)value);
 	}
 
-	else if (XMLString::compareIString(name, XMLUni::fgSAX2XercesSchemaExternalNoNameSpaceSchemaLocation) == 0)
+	else if (XMLString::compareIString(name, XMLUni::fgXercesSchemaExternalNoNameSpaceSchemaLocation) == 0)
 	{
 		fScanner->setExternalNoNamespaceSchemaLocation((XMLCh*)value);
 	}
@@ -1394,9 +1441,9 @@ void SAX2XMLReaderImpl::setProperty(const XMLCh* const name, void* value)
 
 void* SAX2XMLReaderImpl::getProperty(const XMLCh* const name) const
 {
-    if (XMLString::compareIString(name, XMLUni::fgSAX2XercesSchemaExternalSchemaLocation) == 0)
+    if (XMLString::compareIString(name, XMLUni::fgXercesSchemaExternalSchemaLocation) == 0)
         return (void*)fScanner->getExternalSchemaLocation();
-    else if (XMLString::compareIString(name, XMLUni::fgSAX2XercesSchemaExternalNoNameSpaceSchemaLocation) == 0)
+    else if (XMLString::compareIString(name, XMLUni::fgXercesSchemaExternalNoNameSpaceSchemaLocation) == 0)
         return (void*)fScanner->getExternalNoNamespaceSchemaLocation();
     else
         throw SAXNotRecognizedException("Unknown Property");
@@ -1437,3 +1484,87 @@ bool SAX2XMLReaderImpl::getDoSchema() const
 {
     return fScanner->getDoSchema();
 }
+
+
+// ---------------------------------------------------------------------------
+//  SAX2XMLReaderImpl: Grammar preparsing
+// ---------------------------------------------------------------------------
+Grammar* SAX2XMLReaderImpl::loadGrammar(const char* const systemId,
+                                        const short grammarType,
+                                        const bool toCache)
+{
+    // Avoid multiple entrance
+    if (fParseInProgress)
+        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+
+    Grammar* grammar = 0;
+    try
+    {
+        fParseInProgress = true;
+        grammar = fScanner->loadGrammar(systemId, grammarType, toCache);
+        fParseInProgress = false;
+    }
+
+    catch(...)
+    {
+        fParseInProgress = false;
+        throw;
+    }
+
+    return grammar;
+}
+
+Grammar* SAX2XMLReaderImpl::loadGrammar(const XMLCh* const systemId,
+                                        const short grammarType,
+                                        const bool toCache)
+{
+    // Avoid multiple entrance
+    if (fParseInProgress)
+        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+
+    Grammar* grammar = 0;
+    try
+    {
+        fParseInProgress = true;
+        grammar = fScanner->loadGrammar(systemId, grammarType, toCache);
+        fParseInProgress = false;
+    }
+
+    catch(...)
+    {
+        fParseInProgress = false;
+        throw;
+    }
+
+    return grammar;
+}
+
+Grammar* SAX2XMLReaderImpl::loadGrammar(const InputSource& source,
+                                        const short grammarType,
+                                        const bool toCache)
+{
+    // Avoid multiple entrance
+    if (fParseInProgress)
+        ThrowXML(IOException, XMLExcepts::Gen_ParseInProgress);
+
+    Grammar* grammar = 0;
+    try
+    {
+        fParseInProgress = true;
+        grammar = fScanner->loadGrammar(source, grammarType, toCache);
+        fParseInProgress = false;
+    }
+
+    catch(...)
+    {
+        fParseInProgress = false;
+        throw;
+    }
+
+    return grammar;
+}
+
+void SAX2XMLReaderImpl::resetCachedGrammarPool()
+{
+    fScanner->resetCachedGrammarPool();
+}
