@@ -56,6 +56,16 @@
 
 /*
  * $Log: FreeBSDPlatformUtils.cpp,v $
+ * Revision 1.4  2002/07/05 21:29:58  peiyongz
+ * Bug# 10250: patch from James Berry
+ *
+ * Revision 1.3  2002/07/05 21:25:20  peiyongz
+ * Bug# 10250: Implementation of new platform methods,
+ *                      patch from max@cca.usart.ru (Max Gotlib)
+ *
+ * Revision 1.2  2002/05/21 20:31:47  tng
+ * Minor update: Remove obsolete code
+ *
  * Revision 1.1.1.1  2002/02/01 22:22:24  peiyongz
  * sane_include
  *
@@ -120,42 +130,10 @@
 #endif
 
 
-
 // ---------------------------------------------------------------------------
-//  Local Methods
+//  XMLPlatformUtils: Private Static Methods
 // ---------------------------------------------------------------------------
 
-static void WriteCharStr( FILE* stream, const char* const toWrite)
-{
-    if (fputs(toWrite, stream) == EOF)
-    {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::Strm_StdErrWriteFailure);
-    }
-}
-
-static void WriteUStrStdErr( const XMLCh* const toWrite)
-{
-    char* tmpVal = XMLString::transcode(toWrite);
-    ArrayJanitor<char> janText(tmpVal);
-    if (fputs(tmpVal, stderr) == EOF)
-    {
-       ThrowXML(XMLPlatformUtilsException,
-                XMLExcepts::Strm_StdErrWriteFailure);
-   }
-}
-
-static void WriteUStrStdOut( const XMLCh* const toWrite)
- {
-    char* tmpVal = XMLString::transcode(toWrite);
-    ArrayJanitor<char> janText(tmpVal);
-    if (fputs(tmpVal, stdout) == EOF)
-    {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::Strm_StdOutWriteFailure);
-    }
-}
-
 XMLNetAccessor* XMLPlatformUtils::makeNetAccessor()
 {
 #if defined (XML_USE_NETACCESSOR_SOCKET)
@@ -167,6 +145,301 @@ XMLNetAccessor* XMLPlatformUtils::makeNetAccessor()
 
 
 
+//
+//  This method is called by the platform independent part of this class
+//  when client code asks to have one of the supported message sets loaded.
+//
+
+XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
+{
+    XMLMsgLoader* retVal;
+    try
+    {
+#if defined (XML_USE_ICU_MESSAGELOADER)
+        retVal = new ICUMsgLoader(msgDomain);
+#else
+        // same as -DXML_USE_INMEM_MESSAGELOADER
+        retVal = new InMemMsgLoader(msgDomain);
+#endif
+    }
+
+    catch(...)
+    {
+        panic(XMLPlatformUtils::Panic_CantLoadMsgDomain);
+    }
+    return retVal;
+}
+
+
+//
+//  This method is called very early in the bootstrapping process. This guy
+//  must create a transcoding service and return it. It cannot use any string
+//  methods, any transcoding services, throw any exceptions, etc... It just
+//  makes a transcoding service and returns it, or returns zero on failure.
+//
+
+XMLTransService* XMLPlatformUtils::makeTransService()
+{
+#if defined (XML_USE_ICU_TRANSCODER)
+    // Use ICU transcoding services.
+    // same as -DXML_USE_ICU_MESSAGELOADER
+    return new ICUTransService;
+#else
+    // Use native transcoding services.
+    // same as -DXML_USE_INMEM_MESSAGELOADER
+    return new IconvFBSDTransService;
+
+#endif
+}
+
+// ---------------------------------------------------------------------------
+//  XMLPlatformUtils: The panic method
+// ---------------------------------------------------------------------------
+void XMLPlatformUtils::panic(const PanicReasons reason)
+{
+     const char* reasonStr = "Unknown reason";
+    if (reason == Panic_NoTransService)
+        reasonStr = "Could not load a transcoding service";
+    else if (reason == Panic_NoDefTranscoder)
+        reasonStr = "Could not load a local code page transcoder";
+    else if (reason == Panic_CantFindLib)
+        reasonStr = "Could not find the xerces-c DLL";
+    else if (reason == Panic_UnknownMsgDomain)
+        reasonStr = "Unknown message domain";
+    else if (reason == Panic_CantLoadMsgDomain)
+        reasonStr = "Cannot load message domain";
+    else if (reason == Panic_SynchronizationErr)
+        reasonStr = "Cannot synchronize system or mutex";
+    else if (reason == Panic_SystemInit)
+        reasonStr = "Cannot initialize the system or mutex";
+
+    fprintf(stderr, "%s\n", reasonStr);
+
+    exit(-1);
+}
+
+
+// ---------------------------------------------------------------------------
+//  XMLPlatformUtils: File Methods
+// ---------------------------------------------------------------------------
+
+unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile)
+{
+    if (theFile == NULL)
+	ThrowXML(XMLPlatformUtilsException,
+		 XMLExcepts::CPtr_PointerIsZero);
+    off_t curPos = ftell( (FILE*)theFile);
+    if (curPos == -1)
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetSize);
+
+    return (unsigned int)curPos;
+}
+
+void XMLPlatformUtils::closeFile(FileHandle theFile)
+{
+    if (theFile == NULL)
+	ThrowXML(XMLPlatformUtilsException,
+		 XMLExcepts::CPtr_PointerIsZero);
+    if (fclose((FILE*)theFile))
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotCloseFile);
+}
+
+unsigned int XMLPlatformUtils::fileSize(FileHandle theFile)
+{
+    if (theFile == NULL)
+	ThrowXML(XMLPlatformUtilsException,
+		 XMLExcepts::CPtr_PointerIsZero);
+    // Get the current position
+    off_t curPos = ftell((FILE*) theFile);
+    if (curPos == -1)
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotGetCurPos);
+
+    // Seek to the end and save that value for return
+    if (fseek((FILE*) theFile, 0, SEEK_END))
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToEnd);
+
+    off_t retVal = ftell((FILE*)theFile);
+    if (retVal == -1)
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToEnd);
+
+    // And put the pointer back
+    if (fseek( (FILE*)theFile, curPos, SEEK_SET) )
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotSeekToPos);
+
+    return (unsigned int)retVal;
+}
+
+FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
+{
+    if (fileName == NULL)
+	ThrowXML(XMLPlatformUtilsException,
+		 XMLExcepts::CPtr_PointerIsZero);
+    const char* tmpFileName = XMLString::transcode(fileName);
+    ArrayJanitor<char> janText((char*)tmpFileName);
+    FileHandle retVal = (FileHandle)fopen( tmpFileName , "r+" );
+    return retVal;
+}
+
+FileHandle XMLPlatformUtils::openFile(const char* const fileName)
+{
+    if (fileName == NULL)
+	ThrowXML(XMLPlatformUtilsException,
+		 XMLExcepts::CPtr_PointerIsZero);
+    FileHandle retVal = (FileHandle)fopen( fileName , "r+" );
+    return retVal;
+}
+
+FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
+{
+    if (fileName == NULL)
+	ThrowXML(XMLPlatformUtilsException,
+		 XMLExcepts::CPtr_PointerIsZero);
+    const char* tmpFileName = XMLString::transcode(fileName);
+    ArrayJanitor<char> janText((char*)tmpFileName);
+    return fopen( tmpFileName , "w" );
+}
+
+FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName)
+{
+    if (fileName == NULL)
+	ThrowXML(XMLPlatformUtilsException,
+		 XMLExcepts::CPtr_PointerIsZero);
+    return fopen( fileName , "w" );
+}
+
+FileHandle XMLPlatformUtils::openStdInHandle()
+{
+    int nfd = dup(0);
+    if (nfd == -1)
+	ThrowXML(XMLPlatformUtilsException,
+		XMLExcepts::File_CouldNotDupHandle);
+    return (FileHandle) fdopen(dup(0), "r");
+}
+
+unsigned int
+XMLPlatformUtils::readFileBuffer( FileHandle          theFile
+                                , const unsigned int  toRead
+                                , XMLByte* const      toFill)
+{
+    if ( !theFile || !toFill )
+	ThrowXML(XMLPlatformUtilsException,
+		 XMLExcepts::CPtr_PointerIsZero);
+    if (toRead == 0)
+	return 0;
+    size_t noOfItemsRead = fread((void*) toFill, 1, toRead, (FILE*)theFile);
+    
+    if(ferror((FILE*)theFile))
+    {
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotReadFromFile);
+    }
+
+    return (unsigned int)noOfItemsRead;
+}
+
+void XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
+					, long                  toWrite
+					, const XMLByte* const  toFlush )
+{
+    if ( !theFile || !toFlush )
+	ThrowXML(XMLPlatformUtilsException,
+		 XMLExcepts::CPtr_PointerIsZero);
+    if ( toWrite <= 0 )
+	return;
+
+    const XMLByte* tmpFlush = (const XMLByte*) toFlush;
+    size_t bytesWritten = 0;
+
+    while (true)
+    {
+        bytesWritten = fwrite(tmpFlush, sizeof(XMLByte), toWrite, (FILE*)theFile);
+
+        if(ferror((FILE*)theFile))
+	{
+#if 0
+	    ThrowXML(XMLPlatformUtilsException,
+		     XMLExcepts::File_CouldNotWriteToFile);
+#else
+            ThrowXML(XMLPlatformUtilsException,
+		     XMLExcepts::File_CouldNotReadFromFile);
+#endif
+	}
+
+        if (bytesWritten < (size_t) toWrite) //incomplete write
+        {
+            tmpFlush += bytesWritten;
+            toWrite -= bytesWritten;
+            bytesWritten = 0;
+        }
+        else
+            return;
+    }
+}
+
+void XMLPlatformUtils::resetFile(FileHandle theFile)
+{
+    if (theFile == NULL)
+	ThrowXML(XMLPlatformUtilsException,
+		 XMLExcepts::CPtr_PointerIsZero);
+    // Seek to the start of the file
+    if (fseek((FILE*)theFile, 0, SEEK_SET))
+        ThrowXML(XMLPlatformUtilsException,
+                 XMLExcepts::File_CouldNotResetFile);
+}
+
+
+// ---------------------------------------------------------------------------
+//  XMLPlatformUtils: File system methods
+// ---------------------------------------------------------------------------
+XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
+{
+
+    //
+    //  NOTE: THe path provided has always already been opened successfully,
+    //  so we know that its not some pathological freaky path. It comes in
+    //  in native format, and goes out as Unicode always
+    //
+    char* newSrc = XMLString::transcode(srcPath);
+    ArrayJanitor<char> janText(newSrc);
+
+    // Use a local buffer that is big enough for the largest legal path
+    char *absPath = new char[PATH_MAX];
+    // get the absolute path
+    char* retPath = realpath(newSrc, absPath);
+    ArrayJanitor<char> janText2(retPath);
+
+    if (!retPath)
+    {
+        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetBasePathName);
+    }
+    return XMLString::transcode(absPath);
+}
+
+bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
+{
+    // Check for pathological case of empty path
+    if (!toCheck[0])
+        return false;
+
+    //
+    //  If it starts with a slash, then it cannot be relative. This covers
+    //  both something like "\Test\File.xml" and an NT Lan type remote path
+    //  that starts with a node like "\\MyNode\Test\File.xml".
+    //
+    if (toCheck[0] == XMLCh('/'))
+        return false;
+
+    // Else assume its a relative path
+    return true;
+}
+
+
 XMLCh* XMLPlatformUtils::weavePaths(const   XMLCh* const    basePath
                                     , const XMLCh* const    relativePath)
 
@@ -274,186 +547,6 @@ XMLCh* XMLPlatformUtils::weavePaths(const   XMLCh* const    basePath
 }
 
 
-// ---------------------------------------------------------------------------
-//  XMLPlatformUtils: Private Static Methods
-// ---------------------------------------------------------------------------
-
-//
-//  This method is called by the platform independent part of this class
-//  when client code asks to have one of the supported message sets loaded.
-//
-
-XMLMsgLoader* XMLPlatformUtils::loadAMsgSet(const XMLCh* const msgDomain)
-{
-    XMLMsgLoader* retVal;
-    try
-    {
-#if defined (XML_USE_ICU_MESSAGELOADER)
-        retVal = new ICUMsgLoader(msgDomain);
-#else
-        // same as -DXML_USE_INMEM_MESSAGELOADER
-        retVal = new InMemMsgLoader(msgDomain);
-#endif
-    }
-
-    catch(...)
-    {
-        panic(XMLPlatformUtils::Panic_CantLoadMsgDomain);
-    }
-    return retVal;
-}
-
-
-//
-//  This method is called very early in the bootstrapping process. This guy
-//  must create a transcoding service and return it. It cannot use any string
-//  methods, any transcoding services, throw any exceptions, etc... It just
-//  makes a transcoding service and returns it, or returns zero on failure.
-//
-
-XMLTransService* XMLPlatformUtils::makeTransService()
-{
-#if defined (XML_USE_ICU_TRANSCODER)
-    // Use ICU transcoding services.
-    // same as -DXML_USE_ICU_MESSAGELOADER
-    return new ICUTransService;
-#else
-    // Use native transcoding services.
-    // same as -DXML_USE_INMEM_MESSAGELOADER
-    return new IconvFBSDTransService;
-
-#endif
-}
-
-// ---------------------------------------------------------------------------
-//  XMLPlatformUtils: The panic method
-// ---------------------------------------------------------------------------
-void XMLPlatformUtils::panic(const PanicReasons reason)
-{
-     const char* reasonStr = "Unknown reason";
-    if (reason == Panic_NoTransService)
-        reasonStr = "Could not load a transcoding service";
-    else if (reason == Panic_NoDefTranscoder)
-        reasonStr = "Could not load a local code page transcoder";
-    else if (reason == Panic_CantFindLib)
-        reasonStr = "Could not find the xerces-c DLL";
-    else if (reason == Panic_UnknownMsgDomain)
-        reasonStr = "Unknown message domain";
-    else if (reason == Panic_CantLoadMsgDomain)
-        reasonStr = "Cannot load message domain";
-    else if (reason == Panic_SynchronizationErr)
-        reasonStr = "Cannot synchronize system or mutex";
-    else if (reason == Panic_SystemInit)
-        reasonStr = "Cannot initialize the system or mutex";
-
-    fprintf(stderr, "%s\n", reasonStr);
-
-    exit(-1);
-}
-
-
-// ---------------------------------------------------------------------------
-//  XMLPlatformUtils: File Methods
-// ---------------------------------------------------------------------------
-
-unsigned int XMLPlatformUtils::curFilePos(FileHandle theFile)
-{
-    int curPos = ftell( (FILE*)theFile);
-    if (curPos == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotGetSize);
-
-    return (unsigned int)curPos;
-}
-
-void XMLPlatformUtils::closeFile(FileHandle theFile)
-{
-    if (fclose((FILE*)theFile))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotCloseFile);
-}
-
-unsigned int XMLPlatformUtils::fileSize(FileHandle theFile)
-{
-    // Get the current position
-    long  int curPos = ftell((FILE*) theFile);
-    if (curPos == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotGetCurPos);
-
-    // Seek to the end and save that value for return
-     if (fseek((FILE*) theFile, 0, SEEK_END))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToEnd);
-
-    long int retVal = ftell((FILE*)theFile);
-    if (retVal == -1)
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToEnd);
-
-    // And put the pointer back
-
-    if (fseek( (FILE*)theFile, curPos, SEEK_SET) )
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotSeekToPos);
-
-    return (unsigned int)retVal;
-}
-
-FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
-{
-    const char* tmpFileName = XMLString::transcode(fileName);
-    ArrayJanitor<char> janText((char*)tmpFileName);
-    FileHandle retVal = (FILE*)fopen( tmpFileName , "rb" );
-
-    if (retVal == NULL)
-        return 0;
-    return retVal;
-}
-
-FileHandle XMLPlatformUtils::openFile(const char* const fileName)
-{
-    FileHandle retVal = (FILE*)fopen( fileName , "rb" );
-
-    if (retVal == NULL)
-        return 0;
-    return retVal;
-}
-
-
-FileHandle XMLPlatformUtils::openStdInHandle()
-{
-        return (FileHandle)fdopen(dup(0), "rb");
-}
-
-
-
-unsigned int
-XMLPlatformUtils::readFileBuffer( FileHandle          theFile
-                                , const unsigned int  toRead
-                                , XMLByte* const      toFill)
-{
-    size_t noOfItemsRead = fread((void*) toFill, 1, toRead, (FILE*)theFile);
-
-    if(ferror((FILE*)theFile))
-    {
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotReadFromFile);
-    }
-
-    return (unsigned int)noOfItemsRead;
-}
-
-
-void XMLPlatformUtils::resetFile(FileHandle theFile)
-{
-    // Seek to the start of the file
-    if (fseek((FILE*)theFile, 0, SEEK_SET))
-        ThrowXML(XMLPlatformUtilsException,
-                 XMLExcepts::File_CouldNotResetFile);
-}
-
-
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Timing Methods
 // ---------------------------------------------------------------------------
@@ -466,49 +559,6 @@ unsigned long XMLPlatformUtils::getCurrentMillis()
 
 }
 
-XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
-{
-
-    //
-    //  NOTE: THe path provided has always already been opened successfully,
-    //  so we know that its not some pathological freaky path. It comes in
-    //  in native format, and goes out as Unicode always
-    //
-    char* newSrc = XMLString::transcode(srcPath);
-    ArrayJanitor<char> janText(newSrc);
-
-    // Use a local buffer that is big enough for the largest legal path
-    char *absPath = new char[PATH_MAX];
-    // get the absolute path
-    char* retPath = realpath(newSrc, absPath);
-    ArrayJanitor<char> janText2(retPath);
-
-    if (!retPath)
-    {
-        ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotGetBasePathName);
-    }
-    return XMLString::transcode(absPath);
-}
-
-bool XMLPlatformUtils::isRelative(const XMLCh* const toCheck)
-{
-    // Check for pathological case of empty path
-    if (!toCheck[0])
-        return false;
-
-    //
-    //  If it starts with a slash, then it cannot be relative. This covers
-    //  both something like "\Test\File.xml" and an NT Lan type remote path
-    //  that starts with a node like "\\MyNode\Test\File.xml".
-    //
-    if (toCheck[0] == XMLCh('/'))
-        return false;
-
-    // Else assume its a relative path
-    return true;
-}
-
-
 // -----------------------------------------------------------------------
 //  Mutex methods
 // -----------------------------------------------------------------------
@@ -558,7 +608,7 @@ void XMLPlatformUtils::closeMutex(void* const mtxHandle)
             ThrowXML(XMLPlatformUtilsException,
                      XMLExcepts::Mutex_CouldNotDestroy);
         }
-        delete mtxHandle;
+        delete (pthread_mutex_t*)mtxHandle;
     }
 }
 
