@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: OS390PlatformUtils.cpp,v 1.1.1.1 2002/02/01 22:22:25 peiyongz Exp $
+ * $Id: OS390PlatformUtils.cpp,v 1.3 2002/08/08 16:40:35 tng Exp $
  */
 
 
@@ -107,48 +107,6 @@
 #endif
 
 
-// ---------------------------------------------------------------------------
-//  Local Methods
-// ---------------------------------------------------------------------------
-static void WriteCharStr( FILE* stream, const char* const toWrite)
-{
-    if (fputs(toWrite, stream) == EOF)
-	    {
-		ThrowXML(XMLPlatformUtilsException, XMLExcepts::Strm_StdErrWriteFailure);
-    }
-}
-
-static void WriteUStrStdErr( const XMLCh* const toWrite)
-{
-    char* tmpVal = XMLString::transcode(toWrite);
-        ArrayJanitor<char> janText(tmpVal);
-    if (fputs(tmpVal, stderr) == EOF)
-	    {
-		ThrowXML(XMLPlatformUtilsException, XMLExcepts::Strm_StdErrWriteFailure);
-    }
-}
-
-static void WriteUStrStdOut( const XMLCh* const toWrite)
-{
-    char* tmpVal = XMLString::transcode(toWrite);
-        ArrayJanitor<char> janText(tmpVal);
-
-    if (fputs(tmpVal, stdout) == EOF)
-	    {
-		ThrowXML(XMLPlatformUtilsException, XMLExcepts::Strm_StdErrWriteFailure);
-    }
-}
-
-
-XMLNetAccessor* XMLPlatformUtils::makeNetAccessor()
-{
-#if defined (XML_USE_NETACCESSOR_SOCKET)
-    return new SocketNetAccessor();
-#else
-    return 0;
-#endif
-}
-
 // ---------------------------------------------------------------------------
 //  XMLPlatformUtils: Platform init method
 // ---------------------------------------------------------------------------
@@ -167,6 +125,15 @@ void XMLPlatformUtils::platformInit()
 //  XMLPlatformUtils: Private Static Methods
 // ---------------------------------------------------------------------------
 
+XMLNetAccessor* XMLPlatformUtils::makeNetAccessor()
+{
+#if defined (XML_USE_NETACCESSOR_SOCKET)
+    return new SocketNetAccessor();
+#else
+    return 0;
+#endif
+}
+
 //
 //  This method is called by the platform independent part of this class
 //  when client code asks to have one of the supported message sets loaded.
@@ -361,16 +328,16 @@ FileHandle XMLPlatformUtils::openFile(const XMLCh* const fileName)
     }
 #endif
 
-    // fix for file:// protocol                                          
-    // the tmpFileName has a prefix of //absolute path                   
-    // for example, file:////u/.... instead of file:///u/....           
-    // the fopen() on OS/390 cannot open a //u/... POSIX file            
-    if (retVal == NULL) {                                                
-	if ((tmpFileName[0] == '/') && (tmpFileName[1] == '/')) {        
+    // fix for file:// protocol
+    // the tmpFileName has a prefix of //absolute path
+    // for example, file:////u/.... instead of file:///u/....
+    // the fopen() on OS/390 cannot open a //u/... POSIX file
+    if (retVal == NULL) {
+	if ((tmpFileName[0] == '/') && (tmpFileName[1] == '/')) {
 	    char *srcName = tmpFileName + 1; // points past the first '/'
-	    retVal = (FILE*)fopen( srcName , "rb" );                     
-	}                                                                
-    }                                                                     
+	    retVal = (FILE*)fopen( srcName , "rb" );
+	}
+    }
 
     if( retVal == NULL )
 	return 0;
@@ -414,7 +381,7 @@ FileHandle XMLPlatformUtils::openFile(const char* const fileName)
     // Find the last '/' in the path - this seperates the path from the
     // filename.  Then copy the pathname.
 
-    char* pathEnd = strrchr( tmpPos, '/' );
+    char* pathEnd = strrchr((char*) tmpPos, '/' );
     if( pathEnd == NULL ) pathEnd = tmpPos - 1;
     while( tmpPos <= pathEnd ) {
         switch( *tmpPos ) {
@@ -429,7 +396,7 @@ FileHandle XMLPlatformUtils::openFile(const char* const fileName)
 
     // Now we try to locate the extension, and copy that.
 
-    char* extStart = strrchr( fileName, '.' );
+    char* extStart = strrchr((char*) fileName, '.' );
     if ( extStart != NULL ) {
         tmpPos = extStart + 1;
         while( *tmpPos != '\0' ) {
@@ -455,22 +422,173 @@ FileHandle XMLPlatformUtils::openFile(const char* const fileName)
     }
 #endif
 
-    // fix for file:// protocol                                          
-    // the fileName has a prefix of //absolute path                   
-    // for example, file:////u/.... instead of file:///u/....           
-    // the fopen() on OS/390 cannot open a //u/... POSIX file            
-    if (retVal == NULL) {                                                   
-	if ((fileName[0] == '/') && (fileName[1] == '/')) {                 
+    // fix for file:// protocol
+    // the fileName has a prefix of //absolute path
+    // for example, file:////u/.... instead of file:///u/....
+    // the fopen() on OS/390 cannot open a //u/... POSIX file
+    if (retVal == NULL) {
+	if ((fileName[0] == '/') && (fileName[1] == '/')) {
 	    const char *srcName = fileName + 1; // points past the first '/'
-	    retVal = (FILE*)fopen( srcName , "rb" );                        
-	}          	                                                         
-    }                                                                       
-  
+	    retVal = (FILE*)fopen( srcName , "rb" );
+	}          	
+    }
+
     if( retVal == NULL )
         return 0;
     return retVal;
 }
 
+FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
+{
+    const char* tmpFileName = XMLString::transcode(fileName);
+    ArrayJanitor<char> janText((char*)tmpFileName);
+    char* semicolon;
+    char* optStart;
+    int   optOffset = 0;
+    int   dsnSize = 0;
+    int   optSize = 0;
+
+    // Check if this is an MVS dataset. If it is we need to allow
+    // DCB options to be passed in the URI we get. The options
+    // will be after the ";" in the URI in this case.
+    if ((tmpFileName[0] == '/') && (tmpFileName[1] == '/'))
+    {
+       semicolon = strrchr((char*)tmpFileName,';');
+       if (semicolon == NULL)
+          dsnSize = strlen(tmpFileName);
+       else
+       {
+          dsnSize = semicolon - tmpFileName;
+          optStart = semicolon;
+          optStart++;
+          optOffset = optStart - tmpFileName;
+          optSize = (strlen(tmpFileName) - optOffset);
+        }
+
+        // Make a copy of the dataset name to pass to fopen
+        char* dsnbuf = new char[dsnSize + 1];
+        ArrayJanitor<char> janText1((char*)dsnbuf);
+        strncpy(dsnbuf, tmpFileName, dsnSize);
+
+        // If no options specified, then we just put a "wb" in there.
+        // Otherwise we put a "wb" followed by a comma followed by
+        // the options string.
+        if (optSize == 0)
+           optSize += 3;
+        else
+           optSize += 4;
+
+        // Save the options for fopen
+        char* optbuf = new char[optSize];
+        ArrayJanitor<char> janText2((char*)optbuf);
+        strcpy(optbuf, "wb");
+
+        if (optSize != 3)
+        {
+           strcpy(optbuf + 2, ",");
+           strncpy(optbuf +3, optStart, optSize);
+        }
+
+        return fopen(dsnbuf, optbuf);
+
+    }
+    else
+       return fopen(tmpFileName, "wb");
+
+}
+
+FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName)
+{
+    char* semicolon;
+    char* optStart;
+    int   optOffset = 0;
+    int   dsnSize = 0;
+    int   optSize = 0;
+
+    // Check if this is an MVS dataset. If it is we need to allow
+    // DCB options to be passed in the URI we get. The options
+    // will be after the ";" in the URI in this case.
+    if ((fileName[0] == '/') && (fileName[1] == '/'))
+    {
+       semicolon = strrchr((char*)fileName,';');
+       if (semicolon == NULL)
+          dsnSize = strlen(fileName);
+       else
+       {
+          dsnSize = semicolon - fileName;
+          optStart = semicolon;
+          optStart++;
+          optOffset = optStart - fileName;
+          optSize = (strlen(fileName) - optOffset);
+        }
+
+        // Make a copy of the dataset name to pass to fopen
+        char* dsnbuf = new char[dsnSize + 1];
+        ArrayJanitor<char> janText1((char*)dsnbuf);
+        strncpy(dsnbuf, fileName, dsnSize);
+
+        // If no options specified, then we just put a "wb" in there.
+        // Otherwise we put a "wb" followed by a comma followed by
+        // the options string.
+        if (optSize == 0)
+           optSize += 3;
+        else
+           optSize += 4;
+
+        // Save the options for fopen
+        char* optbuf = new char[optSize];
+        ArrayJanitor<char> janText2((char*)optbuf);
+        strcpy(optbuf, "wb");
+
+        if (optSize != 3)
+        {
+           strcpy(optbuf + 2, ",");
+           strncpy(optbuf +3, optStart, optSize);
+        }
+
+        return fopen(dsnbuf, optbuf);
+
+    }
+    else
+       return fopen(fileName, "wb");
+
+}
+
+void
+XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
+                                   , long                  toWrite
+                                   , const XMLByte* const  toFlush)
+{
+    if (!theFile        ||
+        (toWrite <= 0 ) ||
+        !toFlush         )
+        return;
+
+    const XMLByte* tmpFlush = (const XMLByte*) toFlush;
+    size_t bytesWritten = 0;
+
+    while (true)
+    {
+        bytesWritten=fwrite(tmpFlush, sizeof(XMLByte), toWrite, (FILE*)theFile);
+
+        if(ferror((FILE*)theFile))
+        {
+            ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
+        }
+
+        if (bytesWritten < toWrite) //incomplete write
+        {
+            tmpFlush+=bytesWritten;
+            toWrite-=bytesWritten;
+            bytesWritten=0;
+        }
+        else
+            return;
+    }
+
+    return;
+}
+
 unsigned int
 XMLPlatformUtils::readFileBuffer(  FileHandle      theFile
                                 , const unsigned int    toRead
@@ -495,18 +613,9 @@ void XMLPlatformUtils::resetFile(FileHandle theFile)
 }
 
 
-
 // ---------------------------------------------------------------------------
-//  XMLPlatformUtils: Timing Methods
+//  XMLPlatformUtils: File system methods
 // ---------------------------------------------------------------------------
-unsigned long XMLPlatformUtils::getCurrentMillis()
-{
-    timeb aTime;
-    ftime(&aTime);
-    return (unsigned long)(aTime.time*1000 + aTime.millitm);
-
-}
-
 XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
 {
 
@@ -518,6 +627,10 @@ XMLCh* XMLPlatformUtils::getFullPath(const XMLCh* const srcPath)
     char* newSrc = XMLString::transcode(srcPath);
     ArrayJanitor<char> janText(newSrc);
 
+    if ( (newSrc != NULL) && (newSrc[0] != NULL) && (newSrc[1] != NULL) && //@DEM
+         (newSrc[0] == '/') && (newSrc[1] == '/') )       //@DEM
+        return XMLString::transcode(newSrc);              //@DEM
+
     // Use a local buffer that is big enough for the largest legal path
     char *absPath = new char[_POSIX_PATH_MAX];
     ArrayJanitor<char> janText2(absPath);
@@ -675,6 +788,17 @@ XMLCh* XMLPlatformUtils::weavePaths
         return tmpBuf;
 }
 
+// ---------------------------------------------------------------------------
+//  XMLPlatformUtils: Timing Methods
+// ---------------------------------------------------------------------------
+unsigned long XMLPlatformUtils::getCurrentMillis()
+{
+    timeb aTime;
+    ftime(&aTime);
+    return (unsigned long)(aTime.time*1000 + aTime.millitm);
+
+}
+
 // -----------------------------------------------------------------------
 //  Mutex methods
 //  OS390BATCH: Unless POSIX(ON) is specified, pthread functions are
