@@ -55,7 +55,7 @@
  */
 
 /**
- * $Id: OS400PlatformUtils.cpp,v 1.1.1.1 2002/02/01 22:22:25 peiyongz Exp $
+ * $Id: OS400PlatformUtils.cpp,v 1.3 2002/07/16 15:42:47 peiyongz Exp $
  */
 
 // ---------------------------------------------------------------------------
@@ -104,38 +104,8 @@ char* PackingRepText(const char * const repText1,
 		     const char * const repText3,
 		     const char * const repText4);
 // ---------------------------------------------------------------------------
-//  Local Methods
+//  XMLPlatformUtils: Platform init method
 // ---------------------------------------------------------------------------
-
-static void WriteCharStr( FILE* stream, const char* const toWrite)
-{
-    if (fputs(toWrite, stream) == EOF)
-    {
-	ThrowXML(XMLPlatformUtilsException, XMLExcepts::Strm_StdErrWriteFailure);
-    }
-}
-
-
-static void WriteUStrStdErr( const XMLCh* const toWrite)
-{
-    char* tmpVal = XMLString::transcode(toWrite);
-    ArrayJanitor<char> janText(tmpVal);
-    if (fputs(tmpVal, stderr) == EOF)
-    {
-		ThrowXML(XMLPlatformUtilsException, XMLExcepts::Strm_StdErrWriteFailure);
-    }
-}
-
-static void WriteUStrStdOut( const XMLCh* const toWrite)
-{
-
-	char* tmpVal = XMLString::transcode(toWrite);
-    ArrayJanitor<char> janText(tmpVal);
-    if (fputs(tmpVal, stdout) == EOF)
-    {
-		ThrowXML(XMLPlatformUtilsException, XMLExcepts::Strm_StdOutWriteFailure);
-    }
-}
 XMLNetAccessor* XMLPlatformUtils::makeNetAccessor()
 {
     return 0;
@@ -143,9 +113,6 @@ XMLNetAccessor* XMLPlatformUtils::makeNetAccessor()
 
 
 
-// ---------------------------------------------------------------------------
-//  XMLPlatformUtils: Platform init method
-// ---------------------------------------------------------------------------
 //
 //  This method is called very early in the bootstrapping process. This guy
 //  must create a transcoding service and return it. It cannot use any string
@@ -295,6 +262,30 @@ FileHandle XMLPlatformUtils::openFile(const char* const fileName)
     return retVal;
 }
 
+FileHandle XMLPlatformUtils::openFileToWrite(const XMLCh* const fileName)
+{
+    const char* tmpFileName = XMLString::transcode(fileName);
+    ArrayJanitor<char> janText((char*)tmpFileName);
+
+    return openFileToWrite(tmpFileName);
+}
+
+FileHandle XMLPlatformUtils::openFileToWrite(const char* const fileName)
+{
+    char errno_id[7];
+    errno = 0;
+    FileHandle retVal = (FILE*)fopen( fileName , "wb" );
+
+    if (retVal == NULL)
+    {
+     send_message((char*)fileName,FILE_OPEN_PROBLEMS,'d');
+     convert_errno(errno_id,errno);
+     send_message(NULL,errno_id,'d');
+        return 0;
+    }
+
+    return retVal;
+}
 
 unsigned int
 XMLPlatformUtils::readFileBuffer(  FileHandle      theFile
@@ -310,6 +301,40 @@ XMLPlatformUtils::readFileBuffer(  FileHandle      theFile
     return (unsigned int)noOfItemsRead;
 }
 
+void
+XMLPlatformUtils::writeBufferToFile( FileHandle     const  theFile
+                                   , long                  toWrite
+                                   , const XMLByte* const  toFlush)                                   
+{
+    if (!theFile        ||
+        (toWrite <= 0 ) ||
+        !toFlush         )
+        return;
+
+    const XMLByte* tmpFlush = (const XMLByte*) toFlush;
+    size_t bytesWritten = 0;
+
+    while (true)
+    {
+        bytesWritten=fwrite(tmpFlush, sizeof(XMLByte), toWrite, (FILE*)theFile);
+
+        if(ferror((FILE*)theFile))
+        {
+            ThrowXML(XMLPlatformUtilsException, XMLExcepts::File_CouldNotWriteToFile);
+        }
+
+        if (bytesWritten < toWrite) //incomplete write
+        {
+            tmpFlush+=bytesWritten;
+            toWrite-=bytesWritten;
+            bytesWritten=0;
+        }
+        else
+            return;
+    }
+
+    return;
+}
 
 void XMLPlatformUtils::resetFile(FileHandle theFile)
 {
@@ -320,21 +345,9 @@ void XMLPlatformUtils::resetFile(FileHandle theFile)
 
 
 
-
-
 // ---------------------------------------------------------------------------
-//  XMLPlatformUtils: Timing Methods
+//  XMLPlatformUtils: File system methods
 // ---------------------------------------------------------------------------
-unsigned long XMLPlatformUtils::getCurrentMillis()
-{
- _MI_Time mt;
-         struct timeval tv;
-         int rc;
-
-         mattod(mt);
-   rc = Qp0zCvtToTimeval(&tv, mt, QP0Z_CVTTIME_TO_TIMESTAMP);
-   return((tv.tv_sec*1000 )+ (tv.tv_usec/1000));
-}
 /* since we do not have the realpath function on AS/400 and it appears
 to no be important that we convert the name to the real path we will
 only verify that the path exists  - note that this may make AS/400 output a different error for the pathname but customer should
@@ -603,6 +616,20 @@ void abnormal_termination(int termcode)
    send_message(NULL,"CPF9899",'e'); /* send final exception that we have terminated*/
 }
 
+// ---------------------------------------------------------------------------
+//  XMLPlatformUtils: Timing Methods
+// ---------------------------------------------------------------------------
+unsigned long XMLPlatformUtils::getCurrentMillis()
+{
+ _MI_Time mt;
+         struct timeval tv;
+         int rc;
+
+         mattod(mt);
+   rc = Qp0zCvtToTimeval(&tv, mt, QP0Z_CVTTIME_TO_TIMESTAMP);
+   return((tv.tv_sec*1000 )+ (tv.tv_usec/1000));
+}
+
 // -----------------------------------------------------------------------
 //  Mutex methods
 // -----------------------------------------------------------------------
