@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: TransService.cpp,v 1.2 2002/02/05 13:11:45 tng Exp $
+ * $Id: TransService.cpp,v 1.6 2002/07/24 19:10:57 knoaman Exp $
  */
 // ---------------------------------------------------------------------------
 //  Includes
@@ -74,6 +74,7 @@
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/XMLUni.hpp>
 #include <xercesc/util/TransENameMap.hpp>
+#include <xercesc/util/EncodingValidator.hpp>
 
 
 
@@ -98,46 +99,7 @@
 //      we can check for and quickly decide if we need to search the list.
 // ---------------------------------------------------------------------------
 static RefHashTableOf<ENameMap>*    gMappings = 0;
-static XMLCh                        gDisallow1[] =
-{
-        chLatin_I, chLatin_B, chLatin_M, chDash, chDigit_0, chDigit_3
-      , chDigit_7, chNull
-};
-static XMLCh                        gDisallow2[] =
-{
-        chLatin_I, chLatin_B, chLatin_M, chDash, chDigit_3, chDigit_7, chNull
-};
-static XMLCh                        gDisallow3[] =
-{
-        chLatin_I, chLatin_B, chLatin_M, chDash, chDigit_0, chDigit_3
-    ,   chDigit_7, chDash, chLatin_S, chDigit_3, chDigit_9, chDigit_0, chNull
-};
-static XMLCh                        gDisallow4[] =
-{
-        chLatin_I, chLatin_B, chLatin_M, chDash, chDigit_3, chDigit_7, chDash
-    ,   chLatin_S, chDigit_3, chDigit_9, chDigit_0, chNull
-};
-static XMLCh                        gDisallow5[] =
-{
-        chLatin_I, chLatin_B, chLatin_M, chDash, chDigit_1, chDigit_1
-      , chDigit_4, chDigit_0, chNull
-};
-static XMLCh                        gDisallow6[] =
-{
-        chLatin_I, chLatin_B, chLatin_M, chDash, chDigit_1, chDigit_1
-      , chDigit_4, chDigit_0, chDash, chLatin_S, chDigit_3
-      , chDigit_9, chDigit_0, chNull
-};
-static const unsigned int           gDisallowListSize = 6;
-static XMLCh*                       gDisallowList[gDisallowListSize] =
-{
-    gDisallow1, gDisallow2, gDisallow3, gDisallow4, gDisallow5, gDisallow6
-};
-static XMLCh                        gDisallowPre[] =
-{
-        chLatin_I, chLatin_B, chLatin_M, chNull
-};
-
+static bool gStrictIANAEncoding = false;
 
 
 // ---------------------------------------------------------------------------
@@ -185,6 +147,18 @@ XMLTransService::makeNewTranscoderFor(  const   XMLCh* const            encoding
                                         ,       XMLTransService::Codes& resValue
                                         , const unsigned int            blockSize)
 {
+    //
+    // If strict IANA encoding flag is set, validate encoding name
+    //
+    if (gStrictIANAEncoding)
+    {
+        if (!EncodingValidator::instance()->isValidEncoding(encodingName))
+        {
+            resValue = XMLTransService::UnsupportedEncoding;               
+            return 0;
+        }
+    }
+
     //
     //  First try to find it in our list of mappings to intrinsically
     //  supported encodings. We have to upper case the passed encoding
@@ -194,35 +168,34 @@ XMLTransService::makeNewTranscoderFor(  const   XMLCh* const            encoding
     const unsigned int bufSize = 2048;
     XMLCh upBuf[bufSize + 1];
     if (!XMLString::copyNString(upBuf, encodingName, bufSize))
+    {
+        resValue = XMLTransService::InternalFailure;
         return 0;
+    }
     XMLString::upperCase(upBuf);
     ENameMap* ourMapping = gMappings->get(upBuf);
 
     // If we found it, then call the factory method for it
     if (ourMapping)
-        return ourMapping->makeNew(blockSize);
-
-    //
-    //  For now, we have a little list of encodings that we disallow
-    //  explicitly. So lets check for them up front. They all start with
-    //  IBM, so we can do a quick check to see if we should even do
-    //  anything at all.
-    //
-    if (XMLString::startsWith(upBuf, gDisallowPre))
-    {
-        for (unsigned int index = 0; index < gDisallowListSize; index++)
-        {
-            // If its one of our guys, then pretend we don't understand it
-            if (!XMLString::compareString(upBuf, gDisallowList[index]))
-                return 0;
-        }
+	{		
+       XMLTranscoder* temp = ourMapping->makeNew(blockSize);
+       resValue = temp ? XMLTransService::Ok : XMLTransService::InternalFailure;
+       return temp;
     }
 
     //
     //  It wasn't an intrinsic and it wasn't disallowed, so pass it on
     //  to the trans service to see if he can make anything of it.
     //
-    return makeNewXMLTranscoder(encodingName, resValue, blockSize);
+
+    XMLTranscoder* temp =  makeNewXMLTranscoder(encodingName, resValue, blockSize);
+
+    // if successful, set resValue to OK  
+    // if failed, the makeNewXMLTranscoder has already set the proper failing resValue
+    if (temp) resValue =  XMLTransService::Ok;    
+
+    return temp;
+
 }
 
 
@@ -504,3 +477,16 @@ void XMLTransService::initTransService()
     gMappings->put((void*)XMLUni::fgWin1252EncodingString, new ENameMapFor<XMLWin1252Transcoder>(XMLUni::fgWin1252EncodingString));
 
 }
+
+// ---------------------------------------------------------------------------
+//  XLMTransService: IANA encoding setting
+// ---------------------------------------------------------------------------
+void XMLTransService::strictIANAEncoding(const bool newState)
+{
+    gStrictIANAEncoding = newState;
+}
+
+bool XMLTransService::isStrictIANAEncoding()
+{
+    return gStrictIANAEncoding;
+}
