@@ -56,6 +56,12 @@
 
 /*
  * $Log: TransService.hpp,v $
+ * Revision 1.3  2002/07/18 20:05:31  knoaman
+ * Add a new feature to control strict IANA encoding name.
+ *
+ * Revision 1.2  2002/04/09 15:44:00  knoaman
+ * Add lower case string support.
+ *
  * Revision 1.1.1.1  2002/02/01 22:22:13  peiyongz
  * sane_include
  *
@@ -211,6 +217,7 @@ public :
     virtual bool supportsSrcOfs() const = 0;
 
     virtual void upperCase(XMLCh* const toUpperCase) const = 0;
+    virtual void lowerCase(XMLCh* const toLowerCase) const = 0;
 
 	// -----------------------------------------------------------------------
     //	Allow users to add their own encodings to the intrinsinc mapping
@@ -249,6 +256,12 @@ private :
     XMLTransService(const XMLTransService&);
     void operator=(const XMLTransService&);
 
+    // -----------------------------------------------------------------------
+    //  Hidden method to enable/disable strict IANA encoding check
+    //  Caller: XMLPlatformUtils
+    // -----------------------------------------------------------------------
+    void strictIANAEncoding(const bool newState);
+    bool isStrictIANAEncoding();
 
     // -----------------------------------------------------------------------
     //  Hidden init method for platform utils to call
