@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: ICUTransService.cpp,v 1.1.1.1 2002/02/01 22:22:36 peiyongz Exp $
+ * $Id: ICUTransService.cpp,v 1.4 2002/08/27 03:04:19 tng Exp $
  */
 
 
@@ -74,6 +74,9 @@
 #include <unicode/ucnv_err.h>
 #include <unicode/ustring.h>
 #include <unicode/udata.h>
+#if (U_ICU_VERSION_MAJOR_NUM >= 2)
+    #include <unicode/uclean.h>
+#endif
 
 
 #if !defined(XML_OS390) && !defined(XML_AS400) && !defined(XML_HPUX) && !defined(XML_PTX)
@@ -167,6 +170,16 @@ ICUTransService::ICUTransService()
 
 ICUTransService::~ICUTransService()
 {
+    /*
+     * commented out the following clean up code
+     * in case users use ICU outside of the parser
+     * if we clean up here, users' code may crash
+     *
+    #if (U_ICU_VERSION_MAJOR_NUM >= 2)
+        // release all lasily allocated data
+        u_cleanup();
+    #endif
+    */
 }
 
 
@@ -301,6 +314,16 @@ void ICUTransService::upperCase(XMLCh* const toUpperCase) const
     }
 }
 
+void ICUTransService::lowerCase(XMLCh* const toLowerCase) const
+{
+    XMLCh* outPtr = toLowerCase;
+    while (*outPtr)
+    {
+        *outPtr = XMLCh(Unicode::toLowerCase(UChar(*outPtr)));
+        outPtr++;
+    }
+}
+
 
 
 // ---------------------------------------------------------------------------
