@@ -56,6 +56,18 @@
 
 /*
  * $Log: IconvFBSDTransService.cpp,v $
+ * Revision 1.5  2002/07/04 18:20:18  tng
+ * [Bug 10253] Bugfix for the IconvFBSD transcoder.   Patch from Max Gotlib.
+ *
+ * Revision 1.4  2002/04/11 15:38:05  knoaman
+ * String lower case support for FreeBSD by Max Gotlib
+ *
+ * Revision 1.3  2002/04/09 15:44:00  knoaman
+ * Add lower case string support.
+ *
+ * Revision 1.2  2002/03/18 13:39:11  tng
+ * [Bug 7162 ] IconvFreeBSDTransService.cpp needs an #include statement fixed to use xercesc.
+ *
  * Revision 1.1.1.1  2002/02/01 22:22:36  peiyongz
  * sane_include
  *
@@ -96,10 +108,14 @@ typedef struct __IconvFBSDEncoding {
 
 static const IconvFBSDEncoding	gIconvFBSDEncodings[] = {
     { "ucs-2-internal",		2,	LITTLE_ENDIAN },
+    { "ucs2-internal",		2,	LITTLE_ENDIAN },
     { "ucs-4-internal",		4,	LITTLE_ENDIAN },
+    { "ucs4-internal",		4,	LITTLE_ENDIAN },
+    { "UNICODELITTLE",		2,	LITTLE_ENDIAN },
+    { "UNICODEBIG",		2,	BIG_ENDIAN },
     { "iso-10646-ucs-2",	4,	BIG_ENDIAN },
     { "iso-10646-ucs-4",	4,	BIG_ENDIAN },
-    { "iso-10646-utf-16",	2,	BIG_ENDIAN },
+    /* { "iso-10646-utf-16",	2,	BIG_ENDIAN }, */
     { NULL, 0, 	0 }
 };
 
@@ -228,7 +244,7 @@ static size_t fbsd_wcstombs(char *dest, const wchar_t *src, size_t n)
     char*       ptr;
     size_t      slen;
     wchar_t*    wptr;
-    
+
     if (dest)
         return ::wcstombs(dest, src, n);
     if (!src)
@@ -271,17 +287,27 @@ static wint_t fbsd_towupper(wint_t ch)
 {
     if (ch <= 0x7F)
 	return toupper(ch);
-    char	buf[16];
+    unsigned char	buf[16];
     wchar_t	wc = wchar_t(ch);
-    wcstombs(buf, &wc, 16);
+    wcstombs((char*)buf, &wc, 16);
     return toupper(*buf);
 }
 
+static wint_t fbsd_towlower(wint_t ch)
+{
+    if (ch <= 0x7F)
+	return tolower(ch);
+    unsigned char	buf[16];
+    wchar_t	wc = wchar_t(ch);
+    wcstombs((char *)buf, &wc, 16);
+    return tolower(*buf);
+}
+
 #else /* XML_USE_LIBICONV */
 
 #if !defined(APP_NO_THREADS)
 
-#  include <util/Mutexes.hpp>
+#include <xercesc/util/Mutexes.hpp>
 // Iconv() access syncronization point
 static XMLMutex	*gIconvMutex = NULL;
 #  define ICONV_LOCK	XMLMutexLock lockConverter(gIconvMutex);
@@ -339,7 +365,7 @@ void	IconvFBSDCD::mbcToXMLCh (const char *mbc, XMLCh *toRet) const
     }
 }
 
-// Convert XMLCh into "native unicode" character 
+// Convert XMLCh into "native unicode" character
 void	IconvFBSDCD::xmlChToMbc (XMLCh xch, char *mbc) const
 {
     if (fUBO == LITTLE_ENDIAN) {
@@ -366,7 +392,7 @@ XMLCh	IconvFBSDCD::toUpper (const XMLCh ch) const
 {
     if (ch <= 0x7F)
 	return toupper(ch);
-    
+
     char	wcbuf[fUChSize * 2];
     xmlChToMbc (ch, wcbuf);
 
@@ -380,8 +406,40 @@ XMLCh	IconvFBSDCD::toUpper (const XMLCh ch) const
     if (::iconv (fCDTo, (const char**) &ptr, &len,
 		 &pTmpArr, &bLen) == (size_t) -1)
 	return 0;
-    tmpArr[1] = toupper (*tmpArr);
-    *tmpArr = tmpArr[1]; 
+    tmpArr[1] = toupper (*((unsigned char *)tmpArr));
+    *tmpArr = tmpArr[1];
+    len = 1;
+    pTmpArr = wcbuf;
+    bLen = fUChSize;
+    ptr = tmpArr;
+    if (::iconv (fCDFrom, (const char **)&ptr, &len,
+		 &pTmpArr, &bLen) == (size_t) -1)
+	return 0;
+    mbcToXMLCh (wcbuf, (XMLCh*) &ch);
+    return ch;
+}
+
+// Return lowercase equivalent for XMLCh
+XMLCh	IconvFBSDCD::toLower (const XMLCh ch) const
+{
+    if (ch <= 0x7F)
+	return tolower(ch);
+
+    char	wcbuf[fUChSize * 2];
+    xmlChToMbc (ch, wcbuf);
+
+    char	tmpArr[4];
+    char*	ptr = wcbuf;
+    size_t	len = fUChSize;
+    char	*pTmpArr = tmpArr;
+    size_t	bLen = 2;
+
+    ICONV_LOCK;
+    if (::iconv (fCDTo, (const char**) &ptr, &len,
+		 &pTmpArr, &bLen) == (size_t) -1)
+	return 0;
+    tmpArr[1] = tolower (*((unsigned char*)tmpArr));
+    *tmpArr = tmpArr[1];
     len = 1;
     pTmpArr = wcbuf;
     bLen = fUChSize;
@@ -398,6 +456,7 @@ bool	IconvFBSDCD::isSpace(const XMLCh toCheck) const
 {
     if (toCheck <= 0x7F)
 	return isspace(toCheck);
+
     char	wcbuf[fUChSize * 2];
     char	tmpArr[4];
 
@@ -536,7 +595,7 @@ IconvFBSDTransService::IconvFBSDTransService()
 	    XMLPlatformUtils::panic (XMLPlatformUtils::Panic_NoTransService);
     }
 #endif
-    
+
     // Try to obtain local (host) characterset through the environment
     char*	fLocalCP = setlocale (LC_CTYPE, "");
     if (fLocalCP == NULL)
@@ -645,9 +704,9 @@ int IconvFBSDTransService::compareIString(const XMLCh* const	comp1
 	c1 = toUpper(*(++cptr1));
 	c2 = toUpper(*(++cptr2));
 
-    }    
+    }
     return (int) ( c1 - c2 );
-    
+
 #endif /* !XML_USE_LIBICONV */
 }
 
@@ -661,7 +720,7 @@ int IconvFBSDTransService::compareNIString(const XMLCh* const	comp1
     const XMLCh* cptr2 = comp2;
 
 #ifndef XML_USE_LIBICONV
-    
+
     while (true && maxChars)
     {
         wint_t wch1 = fbsd_towupper(*cptr1);
@@ -809,7 +868,7 @@ IconvFBSDTransService::makeNewXMLTranscoder
     if (encLocal)
 	delete [] encLocal;
     return newTranscoder;
-    
+
 #endif /* !XML_USE_LIBICONV */
 }
 
@@ -827,6 +886,19 @@ void IconvFBSDTransService::upperCase(XMLCh* const toUpperCase) const
     }
 }
 
+void IconvFBSDTransService::lowerCase(XMLCh* const toLowerCase) const
+{
+    XMLCh* outPtr = toLowerCase;
+    while (*outPtr)
+    {
+#ifndef XML_USE_LIBICONV
+        *outPtr = fbsd_towlower(*outPtr);
+#else /* XML_USE_LIBICONV */
+	*outPtr = toLower(*outPtr);
+#endif /* !XML_USE_LIBICONV */
+        outPtr++;
+    }
+}
 
 // ---------------------------------------------------------------------------
 //  IconvFBSDLCPTranscoder: The virtual transcoder API
@@ -836,7 +908,7 @@ IconvFBSDLCPTranscoder::calcRequiredSize (const char* const srcText)
 {
     if (!srcText)
         return 0;
-    
+
 #ifndef XML_USE_LIBICONV
 
     unsigned int retVal = fbsd_mbstowcs(NULL, srcText, 0);
@@ -858,8 +930,10 @@ IconvFBSDLCPTranscoder::calcRequiredSize (const char* const srcText)
 	char		*pTmpArr = tmpWideArr;
 	const char	*ptr = srcText + srcLen - len;
 	size_t	rc = iconvFrom(ptr, &len, &pTmpArr, gTempBuffArraySize);
-	if (rc == (size_t) -1 && errno != E2BIG)
-	    return 0;
+	if (rc == (size_t) -1 && errno != E2BIG) {
+	    ThrowXML(TranscodingException, XMLExcepts::Trans_BadSrcSeq);
+	    /* return 0; */
+	}
 	rc = pTmpArr - (char *) tmpWideArr;
 	totalLen += rc;
 	if (rc == 0 || len == 0)
@@ -881,7 +955,7 @@ IconvFBSDLCPTranscoder::calcRequiredSize(const XMLCh* const srcText)
         return 0;
 
 #ifndef XML_USE_LIBICONV
-    
+
     wchar_t       tmpWideCharArr[gTempBuffArraySize];
     wchar_t*      allocatedArray = 0;
     wchar_t*      wideCharBuf = 0;
@@ -920,11 +994,11 @@ IconvFBSDLCPTranscoder::calcRequiredSize(const XMLCh* const srcText)
 	xmlToMbs (srcText, wLent, wBuf, wLent);
     } else
 	wBuf = (char *) srcText;
-    
+
     char	tmpBuff[gTempBuffArraySize];
     size_t	totalLen = 0;
     char	*srcEnd = wBuf + wLent * uChSize();
-    
+
     for (;;) {
 	char		*pTmpArr = tmpBuff;
 	const char	*ptr = srcEnd - len;
@@ -932,7 +1006,8 @@ IconvFBSDLCPTranscoder::calcRequiredSize(const XMLCh* const srcText)
 	if (rc == (size_t) -1 && errno != E2BIG) {
 	    if (wBufPtr)
 		delete [] wBufPtr;
-	    return 0;
+	    ThrowXML(TranscodingException, XMLExcepts::Trans_BadSrcSeq);
+	    /* return 0; */
 	}
 	rc = pTmpArr - tmpBuff;
 	totalLen += rc;
@@ -1110,7 +1185,7 @@ bool IconvFBSDLCPTranscoder::transcode( const   XMLCh* const	toTranscode
     }
     if (wBufPtr)
 	delete [] wBufPtr;
-    
+
 #endif /* !XML_USE_LIBICONV */
 
     // Cap it off just in case
@@ -1274,12 +1349,12 @@ bool IconvFBSDLCPTranscoder::transcode(const   char* const	toTranscode
 	    delete [] wBufPtr;
 	return false;
     }
-    
+
     if (uChSize() != sizeof(XMLCh) || UBO() != BYTE_ORDER)
 	mbsToXML (wideCharBuf, wLent, toFill, wLent);
     if (wBufPtr)
 	delete [] wBufPtr;
-    
+
 #endif /* !XML_USE_LIBICONV */
 
     toFill[wLent] = 0x00;
@@ -1377,7 +1452,7 @@ unsigned int	IconvFBSDTranscoder::transcodeFrom
 	} else
 	    startTarget = tmpWBuff;
     } else
-	startTarget = (char *) toFill;    
+	startTarget = (char *) toFill;
 
     // Do character-by-character transcoding
     char	*orgTarget = startTarget;
@@ -1388,19 +1463,18 @@ unsigned int	IconvFBSDTranscoder::transcodeFrom
     for (size_t cnt = 0; cnt < maxChars && srcLen; cnt++) {
 	size_t	rc = iconvFrom(startSrc, &srcLen, &orgTarget, uChSize());
 	if (rc == (size_t)-1) {
-	    if (errno != E2BIG) {
+	    if (errno != E2BIG || prevSrcLen == srcLen) {
 		if (wBufPtr)
 		    delete [] wBufPtr;
 		ThrowXML(TranscodingException, XMLExcepts::Trans_BadSrcSeq);
 	    }
-	    break;
 	}
 	charSizes[cnt] = prevSrcLen - srcLen;
 	prevSrcLen = srcLen;
 	bytesEaten += charSizes[cnt];
 	startSrc = endSrc - srcLen;
 	toReturn++;
-    }    
+    }
     if (uChSize() != sizeof(XMLCh) || UBO() != BYTE_ORDER)
 	mbsToXML (startTarget, toReturn, toFill, toReturn);
     if (wBufPtr)
