@@ -1,37 +1,37 @@
 /*
  * The Apache Software License, Version 1.1
- * 
+ *
  * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
  * reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 
+ *
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- * 
+ *    notice, this list of conditions and the following disclaimer.
+ *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in
  *    the documentation and/or other materials provided with the
  *    distribution.
- * 
+ *
  * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:  
+ *    if any, must include the following acknowledgment:
  *       "This product includes software developed by the
  *        Apache Software Foundation (http://www.apache.org/)."
  *    Alternately, this acknowledgment may appear in the software itself,
  *    if and wherever such third-party acknowledgments normally appear.
- * 
+ *
  * 4. The names "Xerces" and "Apache Software Foundation" must
  *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written 
+ *    software without prior written permission. For written
  *    permission, please contact apache\@apache.org.
- * 
+ *
  * 5. Products derived from this software may not be called "Apache",
  *    nor may "Apache" appear in their name, without prior written
  *    permission of the Apache Software Foundation.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
  * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
  * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -45,7 +45,7 @@
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  * ====================================================================
- * 
+ *
  * This software consists of voluntary contributions made by many
  * individuals on behalf of the Apache Software Foundation, and was
  * originally based on software copyright (c) 1999, International
@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: Win32TransService.cpp,v 1.1.1.1 2002/02/01 22:22:37 peiyongz Exp $
+ * $Id: Win32TransService.cpp,v 1.4 2002/07/29 15:29:09 tng Exp $
  */
 
 
@@ -406,16 +406,25 @@ Win32TransService::Win32TransService()
             CPMapEntry* aliasedEntry = fCPMap->get(uniAlias);
             if (aliasedEntry)
             {
+                const unsigned int srcLen = strlen(nameBuf);
+                const unsigned int targetLen = ::mbstowcs(0, nameBuf, srcLen);
+                XMLCh* uniName = new XMLCh[targetLen + 1];
+                ::mbstowcs(uniName, nameBuf, srcLen);
+                uniName[targetLen] = 0;
+                _wcsupr(uniName);
+
                 //
                 //  If the name is actually different, then take it.
                 //  Otherwise, don't take it. They map aliases that are
                 //  just different case.
                 //
-                if (::wcscmp(uniAlias, aliasedEntry->getEncodingName()))
+                if (::wcscmp(uniName, aliasedEntry->getEncodingName()))
                 {
-                    CPMapEntry* newEntry = new CPMapEntry(uniAlias, aliasedEntry->getWinCP(), aliasedEntry->getIEEncoding());
+                    CPMapEntry* newEntry = new CPMapEntry(uniName, aliasedEntry->getWinCP(), aliasedEntry->getIEEncoding());
                     fCPMap->put((void*)newEntry->getEncodingName(), newEntry);
                 }
+
+                delete [] uniName;
             }
             delete [] uniAlias;
         }
@@ -489,9 +498,13 @@ void Win32TransService::upperCase(XMLCh* const toUpperCase) const
     _wcsupr(toUpperCase);
 }
 
+void Win32TransService::lowerCase(XMLCh* const toLowerCase) const
+{
+    _wcslwr(toLowerCase);
+}
 
 bool Win32TransService::isAlias(const   HKEY            encodingKey
-                    ,       char* const     aliasBuf 
+                    ,       char* const     aliasBuf
                     , const unsigned int    nameBufSz )
 {
     unsigned long theType;
@@ -640,7 +653,7 @@ Win32Transcoder::transcodeFrom( const   XMLByte* const      srcData
                     , getEncodingName()
                 );
             }
-             else            
+             else
             {
                 ThrowXML(TranscodingException, XMLExcepts::Trans_BadSrcSeq);
             }
