@@ -55,7 +55,7 @@
  */
 
 /*
- * $Id: XMLString.cpp,v 1.1.1.1 2002/02/01 22:22:16 peiyongz Exp $
+ * $Id: XMLString.cpp,v 1.6 2002/08/23 20:55:10 tng Exp $
  */
 
 
@@ -64,6 +64,8 @@
 // ---------------------------------------------------------------------------
 #include <string.h>
 #include <ctype.h>
+#include <stdlib.h>
+#include <errno.h>
 #include <xercesc/util/ArrayIndexOutOfBoundsException.hpp>
 #include <xercesc/util/IllegalArgumentException.hpp>
 #include <xercesc/util/NumberFormatException.hpp>
@@ -1180,14 +1182,19 @@ int XMLString::patternMatch(        XMLCh* const    toSearch
     XMLCh* srcPtr = toSearch;
     const int patnLen = XMLString::stringLen(pattern);
     int  patnIndex = 0;
+    bool patnMatch;
 
     while (*srcPtr)
     {
-        if (*srcPtr++ != pattern[patnIndex++])
+        patnMatch = (*srcPtr++ == pattern[patnIndex++]);
+
+        if (!patnMatch)
         {
             patnIndex = 0;
+            patnMatch = (*srcPtr == pattern[patnIndex++]);
         }
-        else
+
+        if (patnMatch)
         {
             if (patnIndex == patnLen)
                 // full pattern match found
@@ -1317,105 +1324,80 @@ bool XMLString::textToBin(const XMLCh* const toConvert, unsigned int& toFill)
     toFill = 0;
 
     // If no string, then its a failure
-    if (!toConvert)
-        return false;
-    if (!*toConvert)
+    if ((!toConvert) || (!*toConvert))
         return false;
 
-    // Scan past any whitespace. If we hit the end, then return failure
-    const XMLCh* startPtr = toConvert;
-    while (XMLPlatformUtils::fgTransService->isSpace(*startPtr))
-        startPtr++;
-    if (!*startPtr)
-        return false;
+	XMLCh* trimmedStr = XMLString::replicate(toConvert);
+	ArrayJanitor<XMLCh> jan1(trimmedStr);
+	XMLString::trim(trimmedStr);
+    unsigned int trimmedStrLen = XMLString::stringLen(trimmedStr);
 
-    // Start at the end and work back through any whitespace
-    const XMLCh* endPtr = toConvert + stringLen(toConvert);
-    while (XMLPlatformUtils::fgTransService->isSpace(*(endPtr - 1)))
-        endPtr--;
+	if ( !trimmedStrLen )
+		return false;
 
-    //
-    //  Work through what remains and convert each char to a digit. Any
-    //  space or non-digit here is now an error.
-    //
-    unsigned long tmpVal = 0;
-    while (startPtr < endPtr)
-    {
-        // If not valid decimal digit, then an error
-        if ((*startPtr < chDigit_0) || (*startPtr > chDigit_9))
-            return false;
+	// we don't allow '-' sign
+	if (XMLString::indexOf(trimmedStr, chDash, 0) != -1)
+		return false;
 
-        const unsigned int nextVal = (unsigned int)(*startPtr - chDigit_0);
-        tmpVal = (tmpVal * 10) + nextVal;
+	//the errno set by previous run is NOT automatically cleared
+	errno = 0;
 
-        startPtr++;
-    }
+	char *nptr = XMLString::transcode(trimmedStr);
+    ArrayJanitor<char> jan2(nptr);
 
-    // Make sure it didn't overflow
-    if (tmpVal > ~0UL)
-        ThrowXML(RuntimeException, XMLExcepts::Str_ConvertOverflow);
+    char *endptr;
+	 //
+     // REVISIT: conversion of (unsigned long) to (unsigned int)
+	 //          may truncate value on IA64
+    toFill = (unsigned int) strtoul(nptr, &endptr, 10);
+
+	// check if all chars are valid char
+	// check if overflow/underflow occurs
+	if ( ( (endptr - nptr) != (int) trimmedStrLen) ||
+         (errno == ERANGE)                      )
+		return false;
 
-    toFill = (unsigned int)tmpVal;
     return true;
 }
 
 int XMLString::parseInt(const XMLCh* const toConvert)
 {
-    // If no string, then its a failure
-    if ((!toConvert) ||
-        (!*toConvert))
+    // If no string, or empty string, then it is a failure
+    if ((!toConvert) || (!*toConvert))
         ThrowXML(NumberFormatException, XMLExcepts::XMLNUM_null_ptr);
 
-    // Scan past any whitespace. If we hit the end, then return failure
-    const XMLCh* startPtr = toConvert;
-    while (XMLPlatformUtils::fgTransService->isSpace(*startPtr))
-        startPtr++;
+	XMLCh* trimmedStr = XMLString::replicate(toConvert);
+	ArrayJanitor<XMLCh> jan1(trimmedStr);
+	XMLString::trim(trimmedStr);
+    unsigned int trimmedStrLen = XMLString::stringLen(trimmedStr);
 
-    if (!*startPtr)
-        ThrowXML(NumberFormatException, XMLExcepts::CM_UnaryOpHadBinType);
-        //ThrowXML(NumberFormatException, XMLExcepts::XMLINT_Invalid);
+	if ( !trimmedStrLen )
+        ThrowXML(NumberFormatException, XMLExcepts::XMLNUM_null_ptr);
 
-    // Start at the end and work back through any whitespace
-    const XMLCh* endPtr = toConvert + XMLString::stringLen(toConvert);
-    while (XMLPlatformUtils::fgTransService->isSpace(*(endPtr - 1)))
-        endPtr--;
+	//the errno set by previous run is NOT automatically cleared
+	errno = 0;
 
-    //
-    //  Work through what remains and convert each char to a digit.
-    //
-    int signValue = 1;
+	char *nptr = XMLString::transcode(trimmedStr);
+    ArrayJanitor<char> jan2(nptr);
 
-    //
-    // '+' or '-' is allowed only at the first position
-    //
-    if (*startPtr == chDash)
-    {
-        signValue = -1;
-        startPtr++;  // skip the '-'
-    }
-    else if (*startPtr == chPlus)
-        startPtr++;  // skip the '+'
+    char *endptr;
+    long retVal = strtol(nptr, &endptr, 10);
 
-    unsigned long tmpVal = 0;
-    while (startPtr < endPtr)
-    {
-        // If not valid decimal digit, then an error
-        if ((*startPtr < chDigit_0) || (*startPtr > chDigit_9))
-            ThrowXML(NumberFormatException, XMLExcepts::XMLNUM_Inv_chars);
+	// check if all chars are valid char
+	if ( (endptr - nptr) != (int) trimmedStrLen)
+		ThrowXML(NumberFormatException, XMLExcepts::XMLNUM_Inv_chars);
 
-        const unsigned int nextVal = (unsigned int)(*startPtr - chDigit_0);
-        tmpVal = (tmpVal * 10) + nextVal;
-
-        startPtr++;
-    }
-
-    // Make sure it didn't overflow
-    if (tmpVal > ~0UL)
+	// check if overflow/underflow occurs
+    if (errno == ERANGE)
         ThrowXML(NumberFormatException, XMLExcepts::Str_ConvertOverflow);
 
-    return (int) signValue*tmpVal;
+	 //
+     // REVISIT: conversion of (long) to (int)
+	 //          may truncate value on IA64
+	return (int) retVal;
 }
 
+
 void XMLString::trim(XMLCh* const toTrim)
 {
     const unsigned int len = stringLen(toTrim);
@@ -1459,7 +1441,7 @@ void XMLString::upperCase(XMLCh* const toUpperCase)
 void XMLString::lowerCase(XMLCh* const toLowerCase)
 {
     // Refer this one to the transcoding service
-    //XMLPlatformUtils::fgTransService->lowerCase(toLowerCase);
+    XMLPlatformUtils::fgTransService->lowerCase(toLowerCase);
 }
 
 
@@ -1594,7 +1576,7 @@ bool XMLString::isWSReplaced(const XMLCh* const toCheck)
 {
     // If no string, then its a OK
     if (( !toCheck ) || ( !*toCheck ))
-        true;
+        return true;
 
     const XMLCh* startPtr = toCheck;
     while ( *startPtr )
@@ -1656,6 +1638,9 @@ void XMLString::replaceWS(XMLCh* const toConvert)
 //
 bool XMLString::isWSCollapsed(const XMLCh* const toCheck)
 {
+    if (( !toCheck ) || ( !*toCheck ))
+        return true;
+
     // shall be whitespace::replaced first
     if ( !isWSReplaced(toCheck) )
         return false;
