@@ -56,6 +56,9 @@
 
 /*
  * $Log: ParserForXMLSchema.cpp,v $
+ * Revision 1.2  2002/03/18 19:29:53  knoaman
+ * Change constant names to eliminate possible conflict with user defined ones.
+ *
  * Revision 1.1.1.1  2002/02/01 22:22:29  peiyongz
  * sane_include
  *
@@ -151,7 +154,7 @@ Token* ParserForXMLSchema::processQuestion(Token* const tok) {
     TokenFactory* tokFactory = getTokenFactory();
     Token* retTok = tokFactory->createUnion();
     retTok->addChild(tok, tokFactory);
-    retTok->addChild(tokFactory->createToken(Token::EMPTY), tokFactory);
+    retTok->addChild(tokFactory->createToken(Token::T_EMPTY), tokFactory);
     return retTok;
 }
 
